@@ -36,6 +36,7 @@ import tempfile
 import importlib
 import itertools
 import subprocess
+import collections
 import multiprocessing
 from contextlib import contextmanager
 from collections.abc import Mapping, Container, MutableSequence
@@ -54,9 +55,10 @@ mp = multiprocessing.get_context('spawn')
 
 def duplicated(items):
     """
-    :returns: True if the items are duplicated, False otherwise
+    :returns: the list of duplicated keys, possibly empty
     """
-    return len(items) > len(set(items))
+    counter = collections.Counter(items)
+    return [key for key, counts in counter.items() if counts > 1]
 
 
 def cached_property(method):
@@ -896,12 +898,12 @@ def groupby_grid(xs, ys, deltax, deltay):
     xbins = get_bins(xs, nx, None, xmin, xmax)[0]
     ybins = get_bins(ys, ny, None, ymin, ymax)[0]
     acc = AccumDict(accum=[])
-    for p, ij in enumerate(zip(xbins, ybins)):
-        acc[ij].append(p)
+    for k, ij in enumerate(zip(xbins, ybins)):
+        acc[ij].append(k)
     dic = {}
-    for (i, j), ps in acc.items():
-        idxs = numpy.array(ps)
-        dic[xs[idxs].mean(), ys[idxs].mean()] = idxs
+    for ks in acc.values():
+        ks = numpy.array(ks)
+        dic[xs[ks].mean(), ys[ks].mean()] = ks
     return dic
 
 
@@ -1191,23 +1193,6 @@ def random_histogram(counts, nbins, seed):
     return numpy.histogram(numpy.random.random(counts), nbins, (0, 1))[0]
 
 
-def get_indices(integers):
-    """
-    :param integers: a sequence of integers (with repetitions)
-    :returns: a dict integer -> [(start, stop), ...]
-
-    >>> get_indices([0, 0, 3, 3, 3, 2, 2, 0])
-    {0: [(0, 2), (7, 8)], 3: [(2, 5)], 2: [(5, 7)]}
-    """
-    indices = AccumDict(accum=[])  # idx -> [(start, stop), ...]
-    start = 0
-    for i, vals in itertools.groupby(integers):
-        n = sum(1 for val in vals)
-        indices[i].append((start, start + n))
-        start += n
-    return indices
-
-
 def safeprint(*args, **kwargs):
     """
     Convert and print characters using the proper encoding
