@@ -207,7 +207,7 @@ except ImportError:
 
 from openquake.baselib import config, hdf5, workerpool, version
 from openquake.baselib.python3compat import decode
-from openquake.baselib.zeromq import zmq, Socket
+from openquake.baselib.zeromq import zmq, Socket, TimeoutError
 from openquake.baselib.performance import (
     Monitor, memory_rss, init_performance)
 from openquake.baselib.general import (
@@ -258,13 +258,19 @@ def dask_submit(self, func, args, monitor):
         safely_call, func, args, self.task_no, monitor)
 
 
+@submit.add('ipp')
+def ipp_submit(self, func, args, monitor):
+    return self.executor.submit(
+        safely_call, func, args, self.task_no, monitor)
+
+
 def oq_distribute(task=None):
     """
     :returns: the value of OQ_DISTRIBUTE or config.distribution.oq_distribute
     """
     dist = os.environ.get('OQ_DISTRIBUTE', config.distribution.oq_distribute)
     if dist not in ('no', 'processpool', 'threadpool', 'celery', 'zmq',
-                    'dask'):
+                    'dask', 'ipp'):
         raise ValueError('Invalid oq_distribute=%s' % dist)
     return dist
 
@@ -473,7 +479,12 @@ def safely_call(func, args, task_no=0, mon=dummy_mon):
     if mon is dummy_mon:  # in the DbServer
         assert not isgenfunc, func
         return Result.new(func, args, mon)
-    mon = mon.new(operation='total ' + func.__name__, measuremem=True)
+
+    if mon.operation.endswith('_'):
+        name = mon.operation[:-1]
+    else:
+        name = func.__name__
+    mon = mon.new(operation='total ' + name, measuremem=True)
     mon.weight = getattr(args[0], 'weight', 1.)  # used in task_info
     mon.task_no = task_no
     if mon.inject:
@@ -514,6 +525,9 @@ if oq_distribute().startswith('celery'):
 elif oq_distribute() == 'dask':
     from dask.distributed import Client
 
+elif oq_distribute() == 'ipp':
+    from ipyparallel import Cluster
+
 
 class IterResult(object):
     """
@@ -614,15 +628,6 @@ class IterResult(object):
 def init_workers():
     """Waiting function, used to wake up the process pool"""
     setproctitle('oq-worker')
-    # prctl is still useful (on Linux) to terminate all spawned processes
-    # when master is killed via SIGKILL
-    try:
-        import prctl
-    except ImportError:
-        pass
-    else:
-        # if the parent dies, the children die
-        prctl.set_pdeathsig(signal.SIGKILL)
 
 
 def getargnames(task_func):
@@ -660,17 +665,24 @@ class Starmap(object):
             # we use spawn here to avoid deadlocks with logging, see
             # https://github.com/gem/oq-engine/pull/3923 and
             # https://codewithoutrules.com/2018/09/04/python-multiprocessing/
-            cls.pool = multiprocessing.get_context('spawn').Pool(
-                cls.num_cores, init_workers)
+            try:
+                from ray.util.multiprocessing import Pool
+                cls.pool = Pool(cls.num_cores, init_workers)
+            except ImportError:
+                cls.pool = multiprocessing.get_context('spawn').Pool(
+                    cls.num_cores, init_workers)
+                cls.pids = [proc.pid for proc in cls.pool._pool]
             # after spawning the processes restore the original handlers
             # i.e. the ones defined in openquake.engine.engine
             signal.signal(signal.SIGTERM, term_handler)
             signal.signal(signal.SIGINT, int_handler)
-            cls.pids = [proc.pid for proc in cls.pool._pool]
         elif cls.distribute == 'threadpool' and not hasattr(cls, 'pool'):
             cls.pool = multiprocessing.dummy.Pool(cls.num_cores)
         elif cls.distribute == 'dask':
             cls.dask_client = Client(config.distribution.dask_scheduler)
+        elif cls.distribute == 'ipp' and not hasattr(cls, 'executor'):
+            rc = Cluster(n=cls.num_cores).start_and_connect_sync()
+            cls.executor = rc.executor()
 
     @classmethod
     def shutdown(cls):
@@ -684,9 +696,11 @@ class Starmap(object):
             cls.pids = []
         if hasattr(cls, 'dask_client'):
             del cls.dask_client
+        elif hasattr(cls, 'executor'):
+            cls.executor.shutdown()
 
     @classmethod
-    def apply(cls, task, args, concurrent_tasks=None,
+    def apply(cls, task, allargs, concurrent_tasks=None,
               maxweight=None, weight=lambda item: 1,
               key=lambda item: 'Unspecified',
               distribute=None, progress=logging.info, h5=None):
@@ -707,7 +721,7 @@ class Starmap(object):
         :param h5: an open hdf5.File where to store the performance info
         :returns: an :class:`IterResult` object
         """
-        arg0, *args = args
+        arg0, *args = allargs
         if maxweight:  # block_splitter is lazy
             taskargs = ([blk] + args for blk in block_splitter(
                 arg0, maxweight, weight, key))
@@ -718,6 +732,19 @@ class Starmap(object):
                 arg0, concurrent_tasks or 1, weight, key)]
         return cls(task, taskargs, distribute, progress, h5)
 
+    @classmethod
+    def apply_split(cls, task, allargs, concurrent_tasks=None,
+                    maxweight=None, weight=lambda item: 1,
+                    key=lambda item: 'Unspecified',
+                    distribute=None, progress=logging.info, h5=None,
+                    duration=300, outs_per_task=5):
+        """
+        Same as Starmap.apply, but possibly produces subtasks
+        """
+        args = (allargs[0], task, allargs[1:], duration, outs_per_task)
+        return cls.apply(split_task, args, concurrent_tasks or 2*cls.num_cores,
+                         maxweight, weight, key, distribute, progress, h5)
+
     def __init__(self, task_func, task_args=(), distribute=None,
                  progress=logging.info, h5=None):
         self.__class__.init(distribute=distribute)
@@ -732,7 +759,7 @@ class Starmap(object):
         self.monitor = Monitor(task_func.__name__)
         self.monitor.filename = h5.filename
         self.monitor.calc_id = self.calc_id
-        self.name = self.monitor.operation or task_func.__name__
+        self.name = self.monitor.operation
         self.task_args = task_args
         self.progress = progress
         self.h5 = h5
@@ -745,8 +772,7 @@ class Starmap(object):
         self.sent = AccumDict(accum=AccumDict())  # fname -> argname -> nbytes
         self.monitor.inject = (self.argnames[-1].startswith('mon') or
                                self.argnames[-1].endswith('mon'))
-        self.receiver = 'tcp://%s:%s' % (
-            config.dbserver.listen, config.dbserver.receiver_ports)
+        self.receiver = 'tcp://0.0.0.0:%s' % config.dbserver.receiver_ports
         self.monitor.backurl = None  # overridden later
         self.tasks = []  # populated by .submit
         self.task_no = 0
@@ -774,20 +800,19 @@ class Starmap(object):
             self.prev_percent = percent
         return done
 
-    def submit(self, args, func=None, monitor=None):
+    def submit(self, args, func=None):
         """
         Submit the given arguments to the underlying task
         """
-        monitor = monitor or self.monitor
         func = func or self.task_func
         if not hasattr(self, 'socket'):  # first time
             self.t0 = time.time()
             self.__class__.running_tasks = self.tasks
             self.socket = Socket(self.receiver, zmq.PULL, 'bind').__enter__()
-            monitor.backurl = 'tcp://%s:%s' % (
+            self.monitor.backurl = 'tcp://%s:%s' % (
                 config.dbserver.host, self.socket.port)
-            monitor.version = version
-            monitor.config = config
+            self.monitor.version = version
+            self.monitor.config = config
         OQ_TASK_NO = os.environ.get('OQ_TASK_NO')
         if OQ_TASK_NO is not None and self.task_no != int(OQ_TASK_NO):
             self.task_no += 1
@@ -805,10 +830,18 @@ class Starmap(object):
                 fname = func.__name__
                 argnames = getargnames(func)[:-1]
             self.sent[fname] += {a: len(p) for a, p in zip(argnames, args)}
-        res = submit[dist](self, func, args, monitor)
+        res = submit[dist](self, func, args, self.monitor)
         self.task_no += 1
         self.tasks.append(res)
 
+    def submit_split(self, args,  duration, outs_per_task):
+        """
+        Submit the given arguments to the underlying task
+        """
+        self.monitor.operation = self.task_func.__name__ + '_'
+        self.submit((args[0], self.task_func, args[1:], duration, outs_per_task),
+                    split_task)
+
     def submit_all(self):
         """
         :returns: an IterResult object
@@ -910,32 +943,38 @@ def count(word):
     return collections.Counter(word)
 
 
-def split_task(func, *args, duration=1000,
-               weight=operator.attrgetter('weight')):
+class List(list):
+    weight = 0
+
+
+def split_task(elements, func, args, duration, outs_per_task, monitor):
     """
     :param func: a task function with a monitor as last argument
-    :param args: arguments of the task function
+    :param args: arguments of the task function, with args[0] being a sequence
     :param duration: split the task if it exceeds the duration
-    :param weight: weight function for the elements in args[0]
-    :yields: a partial result, 0 or more task objects, 0 or 1 partial result
+    :param outs_per_task: number of splits to try (ex. 5)
+    :yields: a partial result, 0 or more task objects
     """
-    elements = numpy.array(sorted(args[0], key=weight, reverse=True))
     n = len(elements)
-    # print('task_no=%d, num_elements=%d' % (args[-1].task_no, n))
-    assert n > 0, 'Passed an empty sequence!'
-    if n == 1:
-        yield func(*args)
-        return
-    first, *other = elements
-    first_weight = weight(first)
+    if outs_per_task > n:  # too many splits
+        outs_per_task = n
+    elements = numpy.array(elements)  # from WeightedSequence to array
+    idxs = numpy.arange(n)
+    split_elems = [elements[idxs % outs_per_task == i]
+                   for i in range(outs_per_task)]
+    # see how long it takes to run the first slice
     t0 = time.time()
-    res = func(*([first],) + args[1:])
-    dt = (time.time() - t0) / first_weight  # time per unit of weight
-    yield res
-    blocks = list(block_splitter(other, duration, lambda el: weight(el) * dt))
-    for block in blocks[:-1]:
-        yield (func, block) + args[1:-1]
-    yield func(*(blocks[-1],) + args[1:])
+    for i, elems in enumerate(split_elems):
+        res = func(elems, *args, monitor=monitor)
+        dt = time.time() - t0
+        yield res
+        if dt > duration:
+            # spawn subtasks for the rest and exit
+            for els in split_elems[i + 1:]:
+                ls = List(els)
+                ls.weight = sum(getattr(el, 'weight', 1.) for el in els)
+                yield (func, ls) + args
+            break
 
 #                             start/stop workers                             #
 
@@ -1030,6 +1069,8 @@ def workers_status():
     elif OQDIST == 'zmq':
         return workerpool.WorkerMaster().status()
 
+    return []
+
 
 def workers_wait(seconds=30):
     """
