@@ -126,7 +126,7 @@ def compute_disagg(dstore, slc, cmaker, hmap4, magi, bin_edges, monitor):
         for (s, z), r in numpy.ndenumerate(hmap4.rlzs):
             if r in rlzs:
                 g_by_z[s][z] = g
-    eps3 = disagg._eps3(cmaker.trunclevel, cmaker.num_epsilon_bins)
+    eps3 = disagg._eps3(cmaker.truncation_level, cmaker.num_epsilon_bins)
     imts = [from_string(im) for im in cmaker.imtls]
     for magi, ctxs in groupby(allctxs, operator.attrgetter('magi')).items():
         res = {'trti': cmaker.trti, 'magi': magi}
@@ -256,8 +256,9 @@ class DisaggregationCalculator(base.HazardCalculator):
         ws = [rlz.weight for rlz in self.full_lt.get_realizations()]
         dstore = (self.datastore.parent if self.datastore.parent
                   else self.datastore)
+        nrows = len(dstore['_poes/sid'])
         self.pgetter = getters.PmapGetter(
-            dstore, ws, self.sitecol.sids, oq.imtls, oq.poes)
+            dstore, ws, [(0, nrows + 1)], oq.imtls, oq.poes)
 
         # build array rlzs (N, Z)
         if oq.rlz_index is None:
@@ -483,3 +484,14 @@ class DisaggregationCalculator(base.HazardCalculator):
             NML1 = len(vcurves), self.M, oq.imtls.size // self.M
             self.datastore['_vcurves'] = numpy.array(vcurves).reshape(NML1)
             self.datastore['_vcurves'].attrs['sids'] = numpy.where(count)[0]
+
+        # check null realizations in the single site case, see disagg/case_2
+        best_rlzs = self.datastore['best_rlzs'][:]  # (shape N, Z)
+        for (s, z), r in numpy.ndenumerate(best_rlzs):
+            lst = []
+            for key in out:
+                if out[key][s, ..., z].sum() == 0:
+                    lst.append(key)
+            if lst:
+                logging.warning('No %s contributions for site=%d, rlz=%d',
+                                lst, s, r)
