@@ -106,9 +106,8 @@ def main(
     if config_file:
         config.read(os.path.abspath(os.path.expanduser(config_file)),
                     limit=int, soft_mem_limit=int, hard_mem_limit=int,
-                    port=int, multi_user=valid.boolean,
-                    serialize_jobs=valid.boolean, strict=valid.boolean,
-                    code=exec)
+                    port=int, serialize_jobs=valid.boolean,
+                    strict=valid.boolean, code=exec)
 
     if no_distribute:
         os.environ['OQ_DISTRIBUTE'] = 'no'
@@ -118,11 +117,13 @@ def main(
     if not os.path.exists(datadir):
         os.makedirs(datadir)
 
-    dbserver.ensure_on()
-    # check if we are talking to the right server
-    err = dbserver.check_foreign()
-    if err:
-        sys.exit(err)
+    if not os.environ.get('OQ_DATABASE'):
+        # start the dbserver locally if needed
+        dbserver.ensure_on()
+        # check that we are talking to the right server
+        err = dbserver.check_foreign()
+        if err:
+            sys.exit(err)
 
     if upgrade_db:
         msg = logs.dbcmd('what_if_I_upgrade', 'read_scripts')
