@@ -428,123 +428,6 @@ class HazardMapXMLWriter(HazardMapWriter):
             nrml.write(list(root), fh)
 
 
-class DisaggXMLWriter(object):
-    """
-    :param dest:
-        File path (including filename) or file-like object for XML results to
-        be saved to.
-    :param metadata:
-        The following keyword args are required:
-
-        * investigation_time: Investigation time (in years) defined in the
-          calculation which produced these results.
-        * imt: Intensity measure type used to compute these matrices.
-        * lon, lat: Longitude and latitude associated with these results.
-
-        The following attributes define dimension context for the result
-        matrices:
-
-        * mag_bin_edges: List of magnitude bin edges (floats)
-        * dist_bin_edges: List of distance bin edges (floats)
-        * lon_bin_edges: List of longitude bin edges (floats)
-        * lat_bin_edges: List of latitude bin edges (floats)
-        * eps_bin_edges: List of epsilon bin edges (floats)
-        * tectonic_region_types: List of tectonic region types (strings)
-        * smlt_path: String representing the logic tree path which produced
-          these results. Only required for non-statistical results.
-        * gsimlt_path: String represeting the GSIM logic tree path which
-          produced these results. Only required for non-statistical results.
-
-        The following are optional, depending on the `imt`:
-
-        * sa_period
-        * sa_damping
-    """
-
-    #: Maps metadata keywords to XML attribute names for bin edge information
-    #: passed to the constructor.
-    BIN_EDGE_ATTR_MAP = dict([
-        ('mag_bin_edges', 'magBinEdges'),
-        ('dist_bin_edges', 'distBinEdges'),
-        ('lon_bin_edges', 'lonBinEdges'),
-        ('lat_bin_edges', 'latBinEdges'),
-        ('eps_bin_edges', 'epsBinEdges'),
-        ('tectonic_region_types', 'tectonicRegionTypes'),
-    ])
-
-    DIM_LABEL_TO_BIN_EDGE_MAP = dict([
-        ('Mag', 'mag_bin_edges'),
-        ('Dist', 'dist_bin_edges'),
-        ('Lon', 'lon_bin_edges'),
-        ('Lat', 'lat_bin_edges'),
-        ('Eps', 'eps_bin_edges'),
-        ('TRT', 'tectonic_region_types'),
-    ])
-
-    def __init__(self, dest, **metadata):
-        self.dest = dest
-        self.metadata = metadata
-        _validate_hazard_metadata(self.metadata)
-
-    def serialize(self, data):
-        """
-        :param data:
-
-            A sequence of data where each datum has the following attributes:
-
-            * matrix: N-dimensional numpy array containing the disaggregation
-              histogram.
-            * dim_labels: A list of strings which label the dimensions of a
-              given histogram. For example, for a Magnitude-Distance-Epsilon
-              histogram, we would expect `dim_labels` to be
-              ``['Mag', 'Dist', 'Eps']``.
-            * poe: The disaggregation Probability of Exceedance level for which
-              these results were produced.
-            * iml: Intensity measure level, interpolated from the source hazard
-              curve at the given ``poe``.
-        """
-
-        with open(self.dest, 'wb') as fh, floatformat('%.6E'):
-            root = et.Element('nrml')
-
-            diss_matrices = et.SubElement(root, 'disaggMatrices')
-
-            _set_metadata(diss_matrices, self.metadata, _ATTR_MAP)
-
-            transform = lambda val: ', '.join(map(scientificformat, val))
-            _set_metadata(diss_matrices, self.metadata, self.BIN_EDGE_ATTR_MAP,
-                          transform=transform)
-
-            for result in data:
-                diss_matrix = et.SubElement(diss_matrices, 'disaggMatrix')
-
-                # Check that we have bin edges defined for each dimension label
-                # (mag, dist, lon, lat, eps, TRT)
-                for label in result.dim_labels:
-                    bin_edge_attr = self.DIM_LABEL_TO_BIN_EDGE_MAP.get(label)
-                    assert self.metadata.get(bin_edge_attr) is not None, (
-                        "Writer is missing '%s' metadata" % bin_edge_attr
-                    )
-
-                result_type = ','.join(result.dim_labels)
-                diss_matrix.set('type', result_type)
-
-                dims = ','.join(str(x) for x in result.matrix.shape)
-                diss_matrix.set('dims', dims)
-
-                diss_matrix.set('poE', scientificformat(result.poe))
-                diss_matrix.set('iml', scientificformat(result.iml))
-
-                for idxs, value in numpy.ndenumerate(result.matrix):
-                    prob = et.SubElement(diss_matrix, 'prob')
-
-                    index = ','.join(str(x) for x in idxs)
-                    prob.set('index', index)
-                    prob.set('value', scientificformat(value))
-
-            nrml.write(list(root), fh)
-
-
 class UHSXMLWriter(BaseCurveWriter):
     """
     UHS curve XML writer. See :class:`BaseCurveWriter` for a list of general
