@@ -24,13 +24,13 @@ import re
 import sys
 import json
 import time
+import pickle
 import signal
 import getpass
 import logging
 import itertools
 import platform
 from os.path import getsize
-import psutil
 import numpy
 try:
     from setproctitle import setproctitle
@@ -72,7 +72,7 @@ def set_concurrent_tasks_default(calc):
     OqParam.concurrent_tasks.default. Abort the calculations if no
     workers are available. Do nothing for trivial distributions.
     """
-    if OQ_DISTRIBUTE in 'no processpool':  # do nothing
+    if OQ_DISTRIBUTE in 'no processpool ipp':  # do nothing
         num_workers = 0 if OQ_DISTRIBUTE == 'no' else parallel.Starmap.CT // 2
         logging.warning('Using %d cores on %s', num_workers, platform.node())
         return
@@ -109,16 +109,21 @@ def expose_outputs(dstore, owner=getpass.getuser(), status='complete'):
         dskeys.add('realizations')
     hdf5 = dstore.hdf5
     if 'hcurves-stats' in hdf5 or 'hcurves-rlzs' in hdf5:
-        if oq.hazard_stats() or oq.individual_curves or len(rlzs) == 1:
+        if oq.hazard_stats() or oq.individual_rlzs or len(rlzs) == 1:
             dskeys.add('hcurves')
         if oq.uniform_hazard_spectra:
             dskeys.add('uhs')  # export them
         if oq.hazard_maps:
             dskeys.add('hmaps')  # export them
-    if len(rlzs) > 1 and not oq.individual_curves and not oq.collect_rlzs:
-        for out in ['avg_losses-rlzs', 'agg_losses-rlzs', 'agg_curves-rlzs']:
-            if out in dskeys:
-                dskeys.remove(out)
+    if len(rlzs) > 1 and not oq.collect_rlzs:
+        if 'aggrisk' in dstore:
+            dskeys.add('aggrisk-stats')
+        if 'aggcurves' in dstore:
+            dskeys.add('aggcurves-stats')
+        if not oq.individual_rlzs:
+            for out in ['avg_losses-rlzs', 'aggrisk', 'aggcurves']:
+                if out in dskeys:
+                    dskeys.remove(out)
     if 'curves-rlzs' in dstore and len(rlzs) == 1:
         dskeys.add('loss_curves-rlzs')
     if 'curves-stats' in dstore and len(rlzs) > 1:
@@ -209,19 +214,15 @@ def poll_queue(job_id, poll_time):
     if offset >= 0:
         first_time = True
         while True:
-            jobs = logs.dbcmd(GET_JOBS)
-            failed = [job.id for job in jobs if not psutil.pid_exists(job.pid)]
-            if failed:
-                for job in failed:
-                    logs.dbcmd('update_job', job,
-                               {'status': 'failed', 'is_running': 0})
-            elif any(j.id < job_id - offset for j in jobs):
+            running = logs.dbcmd(GET_JOBS)
+            previous = [job for job in running if job.id < job_id - offset]
+            if previous:
                 if first_time:
-                    print('Waiting for jobs %s' % [j.id for j in jobs
-                                                   if j.id < job_id - offset])
                     logs.dbcmd('update_job', job_id,
                                {'status': 'submitted', 'pid': _PID})
                     first_time = False
+                    # the logging is not yet initialized, so use a print
+                    print('Waiting for jobs %s' % [p.id for p in previous])
                 time.sleep(poll_time)
             else:
                 break
@@ -245,8 +246,10 @@ def run_calc(log):
                      calc.oqparam.hazard_calculation_id)
         logging.info('Using engine version %s', __version__)
         msg = check_obsolete_version(oqparam.calculation_mode)
-        if msg:
-            logging.warning(msg)
+        # NB: disabling the warning should be done only for users with
+        # an updated LTS version, but we are doing it for all users
+        # if msg:
+        #    logging.warning(msg)
         calc.from_engine = True
         if config.zworkers['host_cores']:
             set_concurrent_tasks_default(calc)
@@ -298,17 +301,17 @@ def create_jobs(job_inis, log_level=logging.INFO, log_file=None,
         if 'sensitivity_analysis' in dic:
             analysis = valid.dictionary(dic['sensitivity_analysis'])
             for values in itertools.product(*analysis.values()):
-                new = logs.init('job', dic.copy(), log_level, None,
-                                user_name, hc_id)
+                jobdic = dic.copy()
                 pars = dict(zip(analysis, values))
                 for param, value in pars.items():
-                    new.params[param] = str(value)
-                new.params['description'] = '%s %s' % (
-                    dic['description'], pars)
+                    jobdic[param] = str(value)
+                jobdic['description'] = '%s %s' % (dic['description'], pars)
+                new = logs.init('job', jobdic, log_level, log_file,
+                                user_name, hc_id)
                 jobs.append(new)
         else:
             jobs.append(
-                logs.init('job', dic, log_level, None, user_name, hc_id))
+                logs.init('job', dic, log_level, log_file, user_name, hc_id))
     if multi:
         for job in jobs:
             job.multi = True
@@ -336,7 +339,7 @@ def run_jobs(jobs):
             dic = {'status': 'executing', 'pid': _PID}
             logs.dbcmd('update_job', job.calc_id, dic)
     try:
-        if config.zworkers['host_cores'] and parallel.workers_status() == []:
+        if OQ_DISTRIBUTE == 'zmq' and parallel.workers_status() == []:
             print('Asking the DbServer to start the workers')
             logs.dbcmd('workers_start')  # start the workers
         allargs = [(job,) for job in jobs]
@@ -347,7 +350,10 @@ def run_jobs(jobs):
             for job in jobs:
                 run_calc(job)
     finally:
-        if config.zworkers['host_cores']:
+        # for serialize_jobs > 1 there could be something still running:
+        # don't stop the zworkers in that case!
+        if OQ_DISTRIBUTE == 'zmq' and sum(
+                r for h, r, t in parallel.workers_status()) == 0:
             print('Stopping the workers')
             parallel.workers_stop()
     return jobs
@@ -387,20 +393,6 @@ def check_obsolete_version(calculation_mode='WebUI'):
         tag_name = json.loads(decode(data))['tag_name']
         current = version_triple(__version__)
         latest = version_triple(tag_name)
-    except KeyError:  # 'tag_name' not found
-        # NOTE: for unauthenticated requests, the rate limit allows for up
-        # to 60 requests per hour. Therefore, sometimes the api returns the
-        # following message:
-        # b'{"message":"API rate limit exceeded for aaa.aaa.aaa.aaa. (But'
-        # ' here\'s the good news: Authenticated requests get a higher rate'
-        # ' limit. Check out the documentation for more details.)",'
-        # ' "documentation_url":'
-        # ' "https://developer.github.com/v3/#rate-limiting"}'
-        msg = ('An error occurred while calling %s/engine/latest to check'
-               ' if the installed version of the engine is up to date.\n'
-               '%s' % (OQ_API, data.decode('utf8')))
-        logging.warning(msg)
-        return
     except Exception:  # page not available or wrong version tag
         msg = ('An error occurred while calling %s/engine/latest to check'
                ' if the installed version of the engine is up to date.' %
@@ -412,3 +404,12 @@ def check_obsolete_version(calculation_mode='WebUI'):
                 'still using version %s' % (tag_name, __version__))
     else:
         return ''
+
+
+if __name__ == '__main__':
+    from openquake.server import dbserver
+    # run a job object stored in a pickle file, called by job.yaml
+    with open(sys.argv[1], 'rb') as f:
+        job = pickle.load(f)
+    dbserver.ensure_on()
+    run_jobs([job])
