@@ -29,7 +29,6 @@ import numpy
 import scipy.stats
 
 from openquake.baselib.general import AccumDict, groupby, pprod
-from openquake.hazardlib import const
 from openquake.hazardlib.calc import filters
 from openquake.hazardlib.geo.utils import get_longitudinal_extent
 from openquake.hazardlib.geo.utils import (angular_distance, KM_TO_DEGREES,
@@ -61,7 +60,6 @@ def get_edges_shapedic(oq, sitecol, mags_by_trt):
         Z = oq.num_rlzs_disagg or 1
     else:
         Z = len(oq.rlz_index)
-    eps_edges = numpy.linspace(-tl, tl, oq.num_epsilon_bins + 1)
 
     # build mag_edges
     mags = set()
@@ -75,7 +73,7 @@ def get_edges_shapedic(oq, sitecol, mags_by_trt):
         int(numpy.ceil(max(mags) / oq.mag_bin_width) + 1))
 
     # build dist_edges
-    maxdist = max(oq.maximum_distance(trt) for trt in trts)
+    maxdist = max(oq.maximum_distance.max().values())
     dist_edges = oq.distance_bin_width * numpy.arange(
         0, int(numpy.ceil(maxdist / oq.distance_bin_width) + 1))
 
@@ -140,7 +138,7 @@ def disaggregate(ctxs, tom, g_by_z, iml2dict, eps3, sid=0, bin_edges=()):
 
     truncnorm, epsilons, eps_bands = eps3
     cum_bands = numpy.array([eps_bands[e:].sum() for e in range(E)] + [0])
-    G = len(ctxs[0].mean_std)
+    G = ctxs[0].mean_std.shape[1]
     mean_std = numpy.zeros((2, U, M, G), numpy.float32)
     for u, ctx in enumerate(ctxs):
         # search the index associated to the site ID; for instance
@@ -150,7 +148,7 @@ def disaggregate(ctxs, tom, g_by_z, iml2dict, eps3, sid=0, bin_edges=()):
         lons[u] = ctx.clon[idx]  # closest point of the rupture lon
         lats[u] = ctx.clat[idx]  # closest point of the rupture lat
         for g in range(G):
-            mean_std[:, u, :, g] = ctx.mean_std[g][:, :, idx]  # (2, M)
+            mean_std[:, u, :, g] = ctx.mean_std[:, g, :, idx]  # (2, M)
     poes = numpy.zeros((U, E, M, P, Z))
     pnes = numpy.ones((U, E, M, P, Z))
     for (m, p, z), iml in numpy.ndenumerate(iml3):
@@ -177,7 +175,7 @@ def disaggregate(ctxs, tom, g_by_z, iml2dict, eps3, sid=0, bin_edges=()):
 
 def set_mean_std(ctxs, cmaker):
     for u, ctx in enumerate(ctxs):
-        ctx.mean_std = cmaker.get_mean_stds([ctx], const.StdDev.TOTAL)
+        ctx.mean_std = cmaker.get_mean_stds([ctx])[:2]
 
 
 def _disagg_eps(survival, bins, eps_bands, cum_bands):
@@ -263,7 +261,7 @@ def _digitize_lons(lons, lon_bins):
         An instance of `numpy.ndarray`.
     """
     if cross_idl(lon_bins[0], lon_bins[-1]):
-        idx = numpy.zeros_like(lons, dtype=numpy.int)
+        idx = numpy.zeros_like(lons, dtype=int)
         for i_lon in range(len(lon_bins) - 1):
             extents = get_longitudinal_extent(lons, lon_bins[i_lon + 1])
             lon_idx = extents > 0
@@ -371,7 +369,7 @@ def disaggregation(
         cmaker[trt] = ContextMaker(
             trt, rlzs_by_gsim,
             {'truncation_level': truncation_level,
-             'maximum_distance': source_filter.integration_distance,
+             'maximum_distance': source_filter.integration_distance(trt),
              'imtls': {str(imt): [iml]}})
         rups[trt].extend(cmaker[trt].from_srcs(srcs, sitecol))
     min_mag = min(r.mag for rs in rups.values() for r in rs)
