@@ -174,10 +174,9 @@ class HM2018CorrelationModel(BaseCorrelationModel):
 
         See Parent function
         """
-        # stddev_intra is repeated if it is only 1 value for all the residuals
-        if stddev_intra.shape[0] == 1:
-            stddev_intra = numpy.matlib.repmat(
-                stddev_intra, len(sites.complete), 1)
+        # stddev_intra is repeated if there is only one value
+        if len(stddev_intra) == 1:
+            stddev_intra = numpy.full(len(sites.complete), stddev_intra)
         # Reshape 'stddev_intra' if needed
         stddev_intra = stddev_intra.squeeze()
         if not stddev_intra.shape:
