@@ -34,10 +34,12 @@ from openquake.hazardlib.geo.mesh import RectangularMesh
 from openquake.hazardlib.geo import utils as geo_utils
 from openquake.hazardlib.geo.surface import SimpleFaultSurface
 from openquake.hazardlib.geo.surface.base import BaseSurface
-from openquake.hazardlib.geo.geodetic import npoints_towards
-from openquake.hazardlib.geo.geodetic import distance, azimuth
+from openquake.hazardlib.geo.geodetic import (
+    npoints_towards, distance, azimuth)
 
-TOL = 0.2
+TOL = 0.4
+VERY_SMALL = 1e-20
+ALMOST_RIGHT_ANGLE = 89.9
 
 
 def profile_node(points):
@@ -73,16 +75,57 @@ class KiteSurface(BaseSurface):
     composed by several disaligned segments. Thrust faults and listric faults
     can be easily implemented.
     """
-    def __init__(self, mesh, profiles=None):
+
+    def __init__(self, mesh, profiles=None, sec_id=''):
         self.mesh = mesh
+
+        # Clean the mesh
+        success = self._clean()
+        if not success:
+            print('Profiles', profiles)
+            msg = f'Error while initialising section (id: {sec_id})'
+            raise ValueError(msg)
+
+        # Save profiles
         self.profiles = profiles
         assert 1 not in self.mesh.shape, (
-            "Mesh must have at least 2 nodes along both length and width.")
+            "Mesh must have at least 2 nodes along strike and dip.")
+
         # Make sure the mesh respects the right hand rule
         self._fix_right_hand()
         self.strike = self.dip = None
         self.width = None
 
+    def _clean(self):
+        """
+        Removes from the mesh the rows and columns containing just NaNs
+        """
+        # Rows
+        rm = []
+        for i in range(0, self.mesh.lons.shape[0]):
+            if np.all(np.isnan(self.mesh.lons[i, :])):
+                rm.append(i)
+        lons = np.delete(self.mesh.lons, rm, axis=0)
+        lats = np.delete(self.mesh.lats, rm, axis=0)
+        deps = np.delete(self.mesh.depths, rm, axis=0)
+        # Cols
+        rm = []
+        for i in range(0, lons.shape[1]):
+            if np.all(np.isnan(lons[:, i])):
+                rm.append(i)
+        lons = np.delete(lons, rm, axis=1)
+        lats = np.delete(lats, rm, axis=1)
+        deps = np.delete(deps, rm, axis=1)
+
+        success = True
+        if not lons.size > 0:
+            success = False
+            return success
+
+        mesh = RectangularMesh(lons, lats, deps)
+        self.mesh = mesh
+        return success
+
     @property
     def surface_nodes(self):
         """
@@ -92,39 +135,156 @@ class KiteSurface(BaseSurface):
         # from the mesh
         return kite_surface_node(self.profiles)
 
-    def get_joyner_boore_distance(self, mesh):
+    def get_surface_boundaries(self):
+        return self._get_external_boundary()
 
-        # Get indexes of the finite points composing the edges
-        iupp = np.nonzero(np.isfinite(self.mesh.lons[0, :]))[0]
-        ilow = np.flipud(np.nonzero(np.isfinite(self.mesh.lons[-1, :]))[0])
-        irig = np.nonzero(np.isfinite(self.mesh.lons[:, -1]))[0]
-        ilef = np.flipud(np.nonzero(np.isfinite(self.mesh.lons[:, 0]))[0])
+    def get_tor(self):
+        """
+        Provides longitude and latitude coordinates of the vertical surface
+        projection of the top of rupture. This is used in the GC2 method to
+        compute the Rx and Ry0 distances.
 
-        # Building the polygon
-        pnts = []
-        for corner in [(0, iupp), (irig, -1), (-1, ilow), (ilef, 0)]:
-            pnts.extend(zip(self.mesh.lons[corner], self.mesh.lats[corner],
-                            self.mesh.depths[corner]))
-        perimeter = np.array(pnts)
+        One important note here. The kite fault surface uses a rectangular
+        mesh to describe the geometry of the rupture; some nodes can be NaN.
 
+        :returns:
+            Two :class:`numpy.ndarray` instances with the longitudes and
+            latitudes
+        """
+        chk = np.isfinite(self.mesh.lons)
+        iro = (chk).argmax(axis=0)
+        ico = np.arange(0, self.mesh.lons.shape[1])
+        ico = ico[iro <= 1]
+        iro = iro[iro <= 1]
+        return self.mesh.lons[iro, ico], self.mesh.lats[iro, ico]
+
+    def is_vertical(self):
+        """ True if all the profiles, and hence the surface, are vertical """
+        mgd = geodetic.min_geodetic_distance
+        check = []
+        for icol in range(self.mesh.lons.shape[1]):
+            idx = np.isfinite(self.mesh.lons[:, icol])
+            lons = self.mesh.lons[idx, icol]
+            lats = self.mesh.lats[idx, icol]
+            deps = self.mesh.depths[idx, icol]
+            dve = deps[1:] - deps[:-1]
+            dho = mgd((lons[:-1], lats[:-1]), (lons[1:], lats[1:]))
+            tmp = np.ones_like(dve) * 90.0
+            idx = dho > VERY_SMALL
+            tmp[idx] = np.degrees(np.arctan(dve[idx] / dho[idx]))
+            check.append(np.all(tmp > ALMOST_RIGHT_ANGLE))
+        return np.all(check)
+
+    def _get_external_boundary(self):
+        """
+        Provides the surface projection of the external boundary of the
+        rupture surface.
+
+        :returns:
+            Two :class:`numpy.ndarray` instances containing longitudes and
+            latitudes, respectively
+        """
+        if self.is_vertical():
+
+            lo = []
+            la = []
+            idx = np.min(np.where(np.isfinite(self.mesh.lons[:, 0])))
+            slo = self.mesh.lons[idx, 0]
+            sla = self.mesh.lats[idx, 0]
+            idx = np.min(np.where(np.isfinite(self.mesh.lons[:, -1])))
+            elo = self.mesh.lons[idx, -1]
+            ela = self.mesh.lats[idx, -1]
+            strike = azimuth(slo, sla, elo, ela)
+
+            npt = npoints_towards
+            dlt = 0.1
+            tmp = npt(slo, sla, 0.0, strike-90, dlt, 0, 2)
+            lo.append(tmp[0][1])
+            la.append(tmp[1][1])
+            tmp = npt(slo, sla, 0.0, strike+90, dlt, 0, 2)
+            lo.append(tmp[0][1])
+            la.append(tmp[1][1])
+            tmp = npt(elo, ela, 0.0, strike+90, dlt, 0, 2)
+            lo.append(tmp[0][1])
+            la.append(tmp[1][1])
+            tmp = npt(elo, ela, 0.0, strike-90, dlt, 0, 2)
+            lo.append(tmp[0][1])
+            la.append(tmp[1][1])
+
+        else:
+
+            idxs = self._get_external_boundary_indexes()
+            lo = []
+            la = []
+            for i in idxs:
+                lo.append(self.mesh.lons[i[0], i[1]])
+                la.append(self.mesh.lats[i[0], i[1]])
+
+        return np.array(lo), np.array(la)
+
+    def _get_external_boundary_indexes(self):
+        """
+        Computes the indexes of the points composing the boundary of the
+        surface
+        """
+        iul = []
+        ilr = []
+        for i in range(0, self.mesh.lons.shape[1]):
+            idx = np.where(np.isfinite(self.mesh.lons[:, i]))[0]
+            if len(idx) == 0:
+                continue
+            iul.append([min(idx), max(idx)])
+        for i in range(0, self.mesh.lons.shape[0]):
+            idx = np.where(np.isfinite(self.mesh.lons[i, :]))[0]
+            if len(idx) == 0:
+                continue
+            ilr.append([min(idx), max(idx)])
+        iul = np.array(iul)
+        ilr = np.array(ilr)
+        bnd = []
+        # Top
+        for i in range(0, self.mesh.lons.shape[1]):
+            bnd.append([iul[i, 0], i])
+        # Right
+        for i in range(iul[-1, 0]+1, iul[-1, 1]):
+            bnd.append([i, ilr[i, 1]])
+        # Bottom
+        for i in range(self.mesh.lons.shape[1]-1, -1, -1):
+            bnd.append([iul[i, 1], i])
+        # Left
+        for i in range(iul[0, 1]-1, iul[0, 0], -1):
+            bnd.append([i, ilr[i, 0]])
+        return bnd
+
+    def get_joyner_boore_distance(self, mesh) -> np.ndarray:
+        """
+        Computes the Rjb distance between the rupture and the points included
+        in the mesh provided.
+
+        :param mesh:
+            An instance of :class:`openquake.hazardlib.geo.mesh.Mesh`
+        :returns:
+            A :class:`numpy.ndarray` instance with the Rjb values
+        """
+
+        blo, bla = self._get_external_boundary()
         distances = geodetic.min_geodetic_distance(
-            (perimeter[:, 0], perimeter[:, 1]), (mesh.lons, mesh.lats))
+            (blo, bla), (mesh.lons, mesh.lats))
 
         idxs = (distances < 40).nonzero()[0]  # indices on the first dimension
-        if not len(idxs):
+        if len(idxs) < 1:
             # no point is close enough, return distances as they are
             return distances
 
         # Get the projection
         proj = geo_utils.OrthographicProjection(
-            *geo_utils.get_spherical_bounding_box(perimeter[:, 0],
-                                                  perimeter[:, 1]))
+            *geo_utils.get_spherical_bounding_box(blo, bla))
 
         # Mesh projected coordinates
         mesh_xx, mesh_yy = proj(mesh.lons[idxs], mesh.lats[idxs])
 
         # Create the shapely Polygon using projected coordinates
-        xp, yp = proj(perimeter[:, 0], perimeter[:, 1])
+        xp, yp = proj(blo, bla)
         polygon = Polygon([[x, y] for x, y in zip(xp, yp)])
 
         # Calculate the distances
@@ -133,10 +293,11 @@ class KiteSurface(BaseSurface):
 
         return distances
 
-
     def _fix_right_hand(self):
-        # This method fixes the mesh used to represent the grid surface so
-        # that it complies with the right hand rule.
+        """
+        This method fixes the mesh used to represent the grid surface so
+        that it complies with the right hand rule.
+        """
         found = False
         irow = 0
         icol = 0
@@ -171,6 +332,16 @@ class KiteSurface(BaseSurface):
             raise ValueError(msg)
 
     def get_width(self) -> float:
+        # TODO this method is provisional.  It works correctly for simple and
+        # regular geometries defined using profiles parallel to the dip
+        # direction
+        """
+        Compute the width of the kite surface.
+
+        Defining a width for a kite surface is quite difficult. At present we
+        compute it as the mean width for all the columns of the mesh defining
+        the surface.
+        """
         if self.width is None:
             widths = []
             for col_idx in range(self.mesh.lons.shape[1]):
@@ -189,8 +360,11 @@ class KiteSurface(BaseSurface):
         return self.width
 
     def get_dip(self) -> float:
+        # TODO this method is provisional. It works correctly for simple and
+        # regular geometries defined using profiles parallel to the dip
+        # direction
         """
-        Return the fault dip as the average dip over the fault surface mesh.
+        Computes the fault dip as the average dip over the surface.
 
         :returns:
             The average dip, in decimal degrees.
@@ -200,20 +374,30 @@ class KiteSurface(BaseSurface):
             lens = []
             for col_idx in range(self.mesh.lons.shape[1]):
 
-                hdists = distance(self.mesh.lons[:-1, col_idx],
-                                  self.mesh.lats[:-1, col_idx],
-                                  np.zeros_like(self.mesh.depths[1:, col_idx]),
-                                  self.mesh.lons[1:, col_idx],
-                                  self.mesh.lats[1:, col_idx],
-                                  np.zeros_like(self.mesh.depths[1:, col_idx]))
-                vdists = (self.mesh.depths[1:, col_idx] -
-                          self.mesh.depths[:-1, col_idx])
+                # For the calculation of the overall dip we use just the dip
+                # values of contiguous points along a profile
+                iii = np.isfinite(self.mesh.lons[1:, col_idx])
+                kkk = np.isfinite(self.mesh.lons[:-1, col_idx])
+                jjj = np.where(np.logical_and(kkk, iii))[0]
+
+                zeros = np.zeros_like(self.mesh.depths[jjj, col_idx])
+                hdists = distance(self.mesh.lons[jjj+1, col_idx],
+                                  self.mesh.lats[jjj+1, col_idx],
+                                  zeros,
+                                  self.mesh.lons[jjj, col_idx],
+                                  self.mesh.lats[jjj, col_idx],
+                                  zeros)
+                vdists = (self.mesh.depths[jjj+1, col_idx] -
+                          self.mesh.depths[jjj, col_idx])
 
                 ok = np.logical_and(np.isfinite(hdists), np.isfinite(vdists))
                 hdists = hdists[ok]
                 vdists = vdists[ok]
                 if len(vdists) > 0:
-                    dips.append(np.mean(np.degrees(np.arctan(vdists/hdists))))
+                    tmp = np.ones_like(vdists) * 90.
+                    idx = hdists > VERY_SMALL
+                    tmp[idx] = np.degrees(np.arctan(vdists[idx]/hdists[idx]))
+                    dips.append(np.mean(tmp))
                     lens.append(np.sum((hdists**2 + vdists**2)**0.5))
             lens = np.array(lens)
             self.dip = np.sum(np.array(dips) * lens/np.sum(lens))
@@ -250,7 +434,7 @@ class KiteSurface(BaseSurface):
 
     @classmethod
     def from_profiles(cls, profiles, profile_sd, edge_sd, idl=False,
-                      align=False):
+                      align=False, sec_id=''):
         # TODO split this function into smaller components.
         """
         This method creates a quadrilateral mesh from a set of profiles. The
@@ -362,7 +546,7 @@ class KiteSurface(BaseSurface):
         msh = msh.swapaxes(0, 1)
         msh = fix_mesh(msh)
         return cls(RectangularMesh(msh[:, :, 0], msh[:, :, 1], msh[:, :, 2]),
-                   profiles)
+                   profiles, sec_id)
 
     def get_center(self):
         """
@@ -388,87 +572,72 @@ class KiteSurface(BaseSurface):
         :returns:
             Two lists with the coordinates of the longitude and latitude
         """
+        return self._get_external_boundary()
 
-        los = self.mesh.lons
-        las = self.mesh.lats
-        ro, co = np.mgrid[0:los.shape[0], 0:los.shape[1]]
-
-        plos = []
-        plas = []
-
-        tmp = np.amax(ro, where=~np.isnan(los), initial=np.amin(ro)-1, axis=0)
-        for j, i in enumerate(tmp):
-            if i >= np.amin(ro):
-                plos.append(los[i, j])
-                plas.append(las[i, j])
-
-        tlo = []
-        tla = []
-        tmp = np.amin(ro, where=~np.isnan(los), initial=len(los)+1, axis=0)
-        for j, i in enumerate(tmp):
-            if i <= len(los):
-                tlo.append(los[i, j])
-                tla.append(las[i, j])
-
-        plos.extend(tlo[::-1])
-        plas.extend(tla[::-1])
-
-        return plos, plas
+    def get_area(self):
+        _, _, _, cell_area = self.get_cell_dimensions()
+        idx = np.isfinite(cell_area)
+        return np.sum(cell_area[idx])
 
     def get_cell_dimensions(self):
         """
-        Calculate centroid, width, length and area of each mesh cell.
-
-        NOTE: The original verison of this method is in the class
-        :class:`openquake.hazardlib.geo.mesh.Mesh`. It is duplicated here
-        because it required ad-hoc modifications to support kite fault
-        surfaces
-
-        :returns:
-            Tuple of four elements, each being 2d numpy array.
-            Each array has both dimensions less by one the dimensions
-            of the mesh, since they represent cells, not vertices.
-            Arrays contain the following cell information:
-
-            #. centroids, 3d vectors in a Cartesian space,
-            #. length (size along row of points) in km,
-            #. width (size along column of points) in km,
-            #. area in square km.
+        Compute the area [km2] of the cells representing the surface.
         """
-        points, along_azimuth, updip, diag = self.mesh.triangulate()
-        top = along_azimuth[:-1]
-        left = updip[:, :-1]
-        tl_area = geo_utils.triangle_area(top, left, diag)
-        top_length = np.sqrt(np.sum(top * top, axis=-1))
-        left_length = np.sqrt(np.sum(left * left, axis=-1))
+        lo = self.mesh.lons
+        la = self.mesh.lats
+        de = self.mesh.depths
 
-        bottom = along_azimuth[1:]
-        right = updip[:, 1:]
-        br_area = geo_utils.triangle_area(bottom, right, diag)
-        bottom_length = np.sqrt(np.sum(bottom * bottom, axis=-1))
-        right_length = np.sqrt(np.sum(right * right, axis=-1))
+        # Calculating cells dimensions
+        lo0 = lo[:-1, :]
+        la0 = la[:-1, :]
+        de0 = de[:-1, :]
+        lo1 = lo[1:, :]
+        la1 = la[1:, :]
+        de1 = de[1:, :]
+        idx = np.logical_and(np.isfinite(lo0), np.isfinite(lo1))
+        dy = np.full_like(lo0, np.nan)
+        dy[idx] = distance(lo0[idx], la0[idx], de0[idx],
+                           lo1[idx], la1[idx], de1[idx])
 
-        # Remove cells without a finite area
-        np.nan_to_num(tl_area, nan=0.0, copy=False)
-        np.nan_to_num(br_area, nan=0.0, copy=False)
-        cell_area = tl_area + br_area
+        lo0 = lo[:, 1:]
+        la0 = la[:, 1:]
+        de0 = de[:, 1:]
+        lo1 = lo[:, :-1]
+        la1 = la[:, :-1]
+        de1 = de[:, :-1]
+        idx = np.logical_and(np.isfinite(lo0), np.isfinite(lo1))
+        dx = np.full_like(lo0, np.nan)
+        dx[idx] = distance(lo0[idx], la0[idx], de0[idx],
+                           lo1[idx], la1[idx], de1[idx])
 
-        tl_center = (points[:-1, :-1] + points[:-1, 1:] + points[1:, :-1]) / 3
-        br_center = (points[:-1, 1:] + points[1:, :-1] + points[1:, 1:]) / 3
+        lo0 = lo[1:, 1:]
+        la0 = la[1:, 1:]
+        de0 = de[1:, 1:]
+        lo1 = lo[:-1, :-1]
+        la1 = la[:-1, :-1]
+        de1 = de[:-1, :-1]
+        idx = np.logical_and(np.isfinite(lo0), np.isfinite(lo1))
+        dd = np.full_like(lo0, np.nan)
+        dd[idx] = distance(lo0[idx], la0[idx], de0[idx],
+                           lo1[idx], la1[idx], de1[idx])
 
-        cell_center = ((tl_center * tl_area.reshape(tl_area.shape + (1, ))
-                        + br_center * br_area.reshape(br_area.shape + (1, )))
-                       / cell_area.reshape(cell_area.shape + (1, )))
+        # Compute the area of the upper left triangles in each cell
+        s = (dx[:-1, :] + dy[:, :-1] + dd) * 0.5
+        upp = (s * (s - dx[:-1, :]) * (s - dy[:, :-1]) * (s - dd))**0.5
 
-        cell_length = ((top_length * tl_area + bottom_length * br_area)
-                       / cell_area)
-        cell_width = ((left_length * tl_area + right_length * br_area)
-                      / cell_area)
+        # Compute the area of the lower right triangles in each cell
+        s = (dx[1:, :] + dy[:, 1:] + dd) * 0.5
+        low = (s * (s - dx[1:, :]) * (s - dy[:, 1:]) * (s - dd))**0.5
 
-        np.nan_to_num(cell_length, nan=0.0, copy=False)
-        np.nan_to_num(cell_width, nan=0.0, copy=False)
+        # Compute the area of each cell
+        area = np.full_like(dd, np.nan)
+        idx = np.logical_and(np.isfinite(upp), np.isfinite(low))
+        area[idx] = upp[idx] + low[idx]
 
-        return cell_center, cell_length, cell_width, cell_area
+        # Retain the same output of the original function which provided for
+        # each cell the centroid as 3d vector in a Cartesian space, the length
+        # width (size along column of points) in km and the area in km2.
+        return None, None, None, area
 
 
 def get_profiles_from_simple_fault_data(
@@ -519,7 +688,7 @@ def _resample_profile(line, sampling_dist):
     :parameter line:
         An instance of :class:`openquake.hazardlib.geo.line.Line`
     :parameter sampling_dist:
-        A scalar definining the distance [km] used to sample the profile
+        A scalar defining the distance [km] used to sample the profile
     :returns:
         An instance of :class:`openquake.hazardlib.geo.line.Line`
     """
@@ -576,11 +745,12 @@ def _resample_profile(line, sampling_dist):
         # Compute the distance between the starting point and the next point
         # on the profile
         segment_len = distance(slo, sla, sde, lo[idx+1], la[idx+1], de[idx+1])
+        azim = azimuth(slo, sla, lo[idx+1], la[idx+1])
 
-        # Search for the point
+        # Search for the point along the profile
         if cdist+segment_len > sampling_dist:
 
-            # This is the lenght of the last segment-fraction needed to
+            # This is the length of the last segment-fraction needed to
             # obtain the sampling distance
             delta = sampling_dist - cdist
 
@@ -592,12 +762,22 @@ def _resample_profile(line, sampling_dist):
             else:
                 segment_slope = 90.
 
-            # Horizontal and vertical lenght of delta
-            delta_v = delta * np.sin(segment_slope)
-            delta_h = delta * np.cos(segment_slope)
+            # Horizontal and vertical length of delta
+            if segment_slope > ALMOST_RIGHT_ANGLE:
+                delta_v = delta
+                delta_h = 0.0
+            else:
+                delta_v = delta * np.sin(segment_slope)
+                delta_h = delta * np.cos(segment_slope)
 
             # Add a new point to the cross section
-            pnts = npoints_towards(slo, sla, sde, azim, delta_h, delta_v, 2)
+            if segment_slope > ALMOST_RIGHT_ANGLE:
+                pnts = [np.array([slo, slo]),
+                        np.array([sla, sla]),
+                        np.array([sde, sde+delta_v])]
+            else:
+                pnts = npoints_towards(slo, sla, sde,
+                                       azim, delta_h, delta_v, 2)
 
             # Update the starting point
             slo = pnts[0][-1]
@@ -622,7 +802,18 @@ def _resample_profile(line, sampling_dist):
         dst = distance(coo[i, 0], coo[i, 1], coo[i, 2],
                        coo[i+1, 0], coo[i+1, 1], coo[i+1, 2])
         if abs(dst-sampling_dist) > 0.1*sampling_dist:
-            raise ValueError('Wrong distance between points along the profile')
+            msg = 'Distance between points along the profile larger than 10%'
+
+            fmt = '\n   Expected {:.2f} Computed {:.2f}'
+            msg += fmt.format(sampling_dist, dst)
+
+            fmt = '\n   Point {:.2f} {:.2f} {:.2f}'
+            msg += fmt.format(*[coo[i, j] for j in range(3)])
+            msg += fmt.format(*[coo[i+1, j] for j in range(3)])
+
+            msg += '\n   Please, change the sampling distance or the'
+            msg += ' points along the profile'
+            raise ValueError(msg)
 
     return Line(resampled_cs)
 
@@ -721,17 +912,21 @@ def get_mesh(pfs, rfi, sd, idl):
     """
     g = Geod(ellps='WGS84')
 
-    # Residual distance, last index
+    # Instantiate lists with the residual distance and the last profile index
+    # with a finite value at a given depth
     rdist = [0 for _ in range(0, len(pfs[0]))]
     laidx = [0 for _ in range(0, len(pfs[0]))]
+    angle = [0 for _ in range(0, len(pfs[0]))]
 
-    # New profiles
+    # Creating a new list used to collect the new profiles which will describe
+    # the mesh. We start with the initial profile i.e. the one identified by
+    # the reference index rfi
     npr = list([copy.copy(pfs[rfi])])
 
     # Run for all the profiles 'after' the reference one
     for i in range(rfi, len(pfs)-1):
 
-        # Profiles
+        # Profiles: left and right
         pr = pfs[i+1]
         pl = pfs[i]
 
@@ -742,11 +937,14 @@ def get_mesh(pfs, rfi, sd, idl):
                 ptmp = ptmp+360 if ptmp < 0 else ptmp
                 pl[ii][0] = ptmp
 
-        # Point in common on the two profiles
+        # Points in common on the two profiles i.e. points with finite
+        # coordinates on both of them
         cmm = np.logical_and(np.isfinite(pr[:, 2]), np.isfinite(pl[:, 2]))
         cmmi = np.nonzero(cmm)[0].astype(int)
 
-        # Update last profile index
+        # Find the index of the profiles previously analysed and with at least
+        # a node in common with the current profile (i.e. with a continuity in
+        # the mesh)
         mxx = 0
         for ll in laidx:
             if ll is not None:
@@ -755,7 +953,7 @@ def get_mesh(pfs, rfi, sd, idl):
         # Loop over the points in the right profile
         for x in range(0, len(pr[:, 2])):
 
-            # This edge is in common between the last and the current profiles
+            # If true this edge connects the right and left profiles
             if x in cmmi and laidx[x] is None:
                 iii = []
                 for li, lv in enumerate(laidx):
@@ -765,21 +963,36 @@ def get_mesh(pfs, rfi, sd, idl):
                 minidx = np.argmin(abs(iii-x))
                 laidx[x] = mxx
                 rdist[x] = rdist[minidx]
+                angle[x] = angle[minidx]
             elif x not in cmmi:
                 laidx[x] = None
                 rdist[x] = 0
+                angle[x] = None
 
-        # Loop over profiles
+        # Loop over the indexes of the edges in common for the two profiles
+        # starting from the top and going down
         for k in list(np.nonzero(cmm)[0]):
 
-            # Compute distance and azimuth between the corresponding points
-            # on the two profiles
+            # Compute distance [km] and azimuth between the corresponding
+            # points on the two consecutive profiles
             az12, _, hdist = g.inv(pl[k, 0], pl[k, 1], pr[k, 0], pr[k, 1])
             hdist /= 1e3
+            # Vertical distance
             vdist = pr[k, 2] - pl[k, 2]
+            # Total distance
             tdist = (vdist**2 + hdist**2)**.5
-            ndists = int(np.floor((tdist+rdist[k])/sd))
 
+            # Update rdist
+            new_rdist = rdist[k]
+            if rdist[k] > 0 and abs(az12 - angle[k] > 2):
+                new_rdist = update_rdist(rdist[k], az12, angle[k], sd)
+
+            # Number of grid points
+            # ndists = int(np.floor((tdist+rdist[k])/sd))
+            ndists = int(np.floor((tdist+new_rdist)/sd))
+
+            # Calculate points between the corresponding nodes on the
+            # two profiles
             ll = g.npts(pl[k, 0], pl[k, 1], pr[k, 0], pr[k, 1],
                         np.ceil(tdist)*20)
             ll = np.array(ll)
@@ -793,12 +1006,14 @@ def get_mesh(pfs, rfi, sd, idl):
             tdsts = (hdsts**2 + (pl[k, 2]-deps)**2)**0.5
             assert len(deps) == ll.shape[0]
 
-            # Compute distance between consecutive profiles
+            # Compute distance between nodels at depth 'k' on the two
+            # consecutive profiles
             dd = distance(pl[k, 0], pl[k, 1], pl[k, 2],
                           pr[k, 0], pr[k, 1], pr[k, 2])
 
-            # Check distance
-            if abs(dd-tdist) > 0.1*tdist:
+            # Check that the actual distance between these nodes is similar to
+            # the one originally defined
+            if abs(dd - tdist) > 0.1*tdist:
                 print('dd:', dd)
                 tmps = 'Error while building the mesh'
                 tmps += '\nDistances: {:f} {:f}'
@@ -807,21 +1022,27 @@ def get_mesh(pfs, rfi, sd, idl):
             # Adding new points along the edge with index k
             for j in range(ndists):
 
-                # Add new profile
+                # Add new profile to 'npr' i.e. the list containing the new
+                # set of profiles
                 if len(npr)-1 < laidx[k]+1:
                     npr = add_empty_profile(npr)
 
                 # Compute the coordinates of intermediate points along the
-                # current edge
-                tmp = (j+1)*sd - rdist[k]
-                lo, la, _ = g.fwd(pl[k, 0], pl[k, 1], az12,
-                                  tmp*hdist/tdist*1e3)
+                # current edge. 'tmp' is the distance between the node on the
+                # left edge and the j-th node on the edge. 'lo' and 'la' are
+                # the coordinates of this new node
+                # tmp = (j+1)*sd - rdist[k]
+                tmp = (j+1)*sd - new_rdist
+                # lo, la, _ = g.fwd(pl[k, 0], pl[k, 1], az12,
+                #                  tmp*hdist/tdist*1e3)
 
+                # Find the index of the closest node in the vector sampled at
+                # high frequency
                 tidx = np.argmin(abs(tdsts-tmp))
                 lo = ll[tidx, 0]
                 la = ll[tidx, 1]
 
-                # Fix longitudes
+                # Fix longitudes in proximity of the IDL
                 if idl:
                     lo = lo+360 if lo < 0 else lo
 
@@ -829,28 +1050,67 @@ def get_mesh(pfs, rfi, sd, idl):
                 de = pl[k, 2] + tmp*vdist/hdist
                 de = deps[tidx]
 
+                # Updating the new profile
                 npr[laidx[k]+1][k] = [lo, la, de]
                 if (k > 0 and np.all(np.isfinite(npr[laidx[k]+1][k])) and
                         np.all(np.isfinite(npr[laidx[k]][k]))):
 
+                    # Computing the distance between consecutive points on
+                    # one edge
                     p1 = npr[laidx[k]][k]
                     p2 = npr[laidx[k]+1][k]
                     d = distance(p1[0], p1[1], p1[2], p2[0], p2[1], p2[2])
 
-                    # Check
-                    if abs(d-sd) > 0.1*sd:
-                        tmpf = 'd: {:f} diff: {:f} tol: {:f} sd:{:f}'
-                        tmpf += '\nresidual: {:f}'
-                        tmps = tmpf.format(d, d-sd, TOL*sd, sd, rdist[k])
+                    # Check if the distance between consecutive points on one
+                    # edge (with index k) is within a tolerance limit of the
+                    # mesh distance defined by the user
+                    if abs(d-sd) > TOL*sd:
+                        tmpf = '\ndistance: {:f} difference: {:f} '
+                        tmpf += '\ntolerance dist: {:f} sampling dist: {:f}'
+                        tmpf += '\nresidual distance: {:f}'
+                        tmps = tmpf.format(d, d-sd, TOL*sd, sd, new_rdist)
                         raise ValueError(tmps)
                 laidx[k] += 1
 
-            rdist[k] = tdist - sd*ndists + rdist[k]
+            # Check that the residual distance along each edge is lower than
+            # the sampling distance
+            rdist[k] = tdist - sd*ndists + new_rdist
+            angle[k] = az12
             assert rdist[k] < sd
 
     return npr
 
 
+def update_rdist(rdist, az12, angle, sd):
+    r"""
+    Here we adjust the residual distance to make sure that the size of the
+    mesh is consistent with the sampling. This is particularly needed when the
+    mesh has a kink
+
+    v1
+    ------------------------------  angle[k]
+             beta  \ az12-angle[k]
+                    \
+                     \
+                      \ v2
+
+    alpha is the angle between angle[k] and the v1 to v2 direction
+    gamma is the angle between az12 (the new azimuth) and v1-v2 i.e.
+    the third angle in the triangle
+    """
+    assert rdist > 0
+    beta = 180.0 - abs(az12 - angle)
+    side_b = sd
+    side_a = (sd - rdist)
+    ratio = side_b / np.sin(np.radians(beta))
+    alpha = np.rad2deg(np.arcsin(side_a / ratio))
+    gamma = 180.0 - (alpha + beta)
+    assert gamma > 0
+    rdist_new = ratio * np.sin(np.radians(gamma))
+    assert (rdist_new) > 0
+    return rdist_new
+
+
 def get_mesh_back(pfs, rfi, sd, idl):
     """
     Compute resampled profiles in the backward direction from the reference
@@ -875,6 +1135,7 @@ def get_mesh_back(pfs, rfi, sd, idl):
     # Initialize residual distance and last index lists
     rdist = [0 for _ in range(0, len(pfs[0]))]
     laidx = [0 for _ in range(0, len(pfs[0]))]
+    angle = [0 for _ in range(0, len(pfs[0]))]
 
     # Create list containing the new profiles. We start by adding the
     # reference profile
@@ -891,16 +1152,20 @@ def get_mesh_back(pfs, rfi, sd, idl):
         # profiles are not NaN
         cmm = np.logical_and(np.isfinite(pr[:, 2]), np.isfinite(pl[:, 2]))
 
-        # Transform the indexes into integers and initialise the maximum
-        # index of the points in common
+        # Transform the 'cmm' indexes into integers and calculate the index of
+        # the last valid profile i.e. the index of the closest one to the
+        # current left profile
         cmmi = np.nonzero(cmm)[0].astype(int)
         mxx = 0
         for ll in laidx:
             if ll is not None:
                 mxx = max(mxx, ll)
 
-        # Update indexes
+        # For each edge in the right profile we compute
         for x in range(0, len(pr[:, 2])):
+
+            # If this index is in cmmi and last index is None the mesh at this
+            # depth starts from this profile
             if x in cmmi and laidx[x] is None:
                 iii = []
                 for li, lv in enumerate(laidx):
@@ -910,9 +1175,11 @@ def get_mesh_back(pfs, rfi, sd, idl):
                 minidx = np.argmin(abs(iii-x))
                 laidx[x] = mxx
                 rdist[x] = rdist[minidx]
+                angle[x] = angle[minidx]
             elif x not in cmmi:
                 laidx[x] = None
                 rdist[x] = 0
+                angle[x] = None
 
         # Loop over the points in common between the two profiles
         for k in list(np.nonzero(cmm)[0]):
@@ -922,7 +1189,16 @@ def get_mesh_back(pfs, rfi, sd, idl):
             hdist /= 1e3
             vdist = pr[k, 2] - pl[k, 2]
             tdist = (vdist**2 + hdist**2)**.5
-            ndists = int(np.floor((tdist+rdist[k])/sd))
+
+            # Update rdist if this is larger than 0 and the new edge has a
+            # different direction than the previous one
+            new_rdist = rdist[k]
+            if rdist[k] > 0 and abs(az12 - angle[k] > 2):
+                new_rdist = update_rdist(rdist[k], az12, angle[k], sd)
+
+            # Calculate the number of cells
+            #ndists = int(np.floor((tdist+rdist[k])/sd))
+            ndists = int(np.floor((tdist+new_rdist)/sd))
 
             # Adding new points along edge with index k
             for j, _ in enumerate(range(ndists)):
@@ -932,7 +1208,8 @@ def get_mesh_back(pfs, rfi, sd, idl):
                     npr = add_empty_profile(npr)
                 #
                 # fix distance
-                tmp = (j+1)*sd - rdist[k]
+                #tmp = (j+1)*sd - rdist[k]
+                tmp = (j+1)*sd - new_rdist
                 lo, la, _ = g.fwd(pl[k, 0], pl[k, 1], az12,
                                   tmp*hdist/tdist*1e3)
 
@@ -948,18 +1225,28 @@ def get_mesh_back(pfs, rfi, sd, idl):
                     p1 = npr[laidx[k]][k]
                     p2 = npr[laidx[k]+1][k]
                     d = distance(p1[0], p1[1], p1[2], p2[0], p2[1], p2[2])
-                    #
-                    # >>> TOLERANCE
+
+                    # This checks that the size of each newly created cell
+                    # is similar (within some tolerance) to the intial mesh
+                    # size provided by the user
                     if abs(d-sd) > TOL*sd:
                         tmpf = 'd: {:f} diff: {:f} tol: {:f} sd:{:f}'
                         tmpf += '\nresidual: {:f}'
-                        tmps = tmpf.format(d, d-sd, TOL*sd, sd, rdist[k])
+                        tmps = tmpf.format(d, d-sd, TOL*sd, sd, new_rdist)
                         msg = 'The mesh spacing exceeds the tolerance limits'
                         tmps += '\n {:s}'.format(msg)
                         raise ValueError(tmps)
 
+                # Updating the index of the last profile in the mesh at depth
+                # 'k'
                 laidx[k] += 1
-            rdist[k] = tdist - sd*ndists + rdist[k]
+
+            # Updating residual distances and angle (i.e. azimuth)
+            rdist[k] = tdist - sd*ndists + new_rdist
+            angle[k] = az12
+
+            # Checking that the residual distance is lower than the sampling
+            # distance
             assert rdist[k] < sd
 
     tmp = []
