@@ -20,12 +20,13 @@
 Module :mod:`openquake.hazardlib.geo.surface.multi` defines
 :class:`MultiSurface`.
 """
-import numpy
 from copy import deepcopy
+import numpy
+from shapely.geometry import Polygon
 from scipy.spatial.distance import pdist, squareform
 from openquake.baselib.hdf5 import read_csv
 from openquake.hazardlib.geo.surface.base import BaseSurface, downsample_trace
-from openquake.hazardlib.geo.mesh import Mesh
+from openquake.hazardlib.geo.mesh import Mesh, RectangularMesh
 from openquake.hazardlib.geo import utils
 from openquake.hazardlib import geo
 from openquake.hazardlib.geo.surface import (
@@ -113,25 +114,23 @@ class MultiSurface(BaseSurface):
         """
         if type(self.surfaces[0]).__name__ == 'PlanarSurface':
             return [surf.surface_nodes[0] for surf in self.surfaces]
-        else:
-            return [surf.surface_nodes for surf in self.surfaces]
+        return [surf.surface_nodes for surf in self.surfaces]
 
     @property
     def mesh(self):
         """
         :returns: mesh corresponding to the whole multi surface
         """
-        meshes = [surface.mesh for surface in self.surfaces]
         lons = []
         lats = []
         deps = []
-        for m in meshes:
-            for lo, la, de in zip(m.lons, m.lats, m.depths):
-                if numpy.isfinite(lo) and numpy.isfinite(la):
-                    lons.append(lo)
-                    lats.append(la)
-                    deps.append(de)
-        return Mesh(numpy.array(lons), numpy.array(lats), numpy.array(deps))
+        for m in [surface.mesh for surface in self.surfaces]:
+            ok = numpy.isfinite(m.lons) & numpy.isfinite(m.lats)
+            lons.append(m.lons[ok])
+            lats.append(m.lats[ok])
+            deps.append(m.depths[ok])
+        return Mesh(numpy.concatenate(lons), numpy.concatenate(lats),
+                    numpy.concatenate(deps))
 
     def __init__(self, surfaces, tol=0.1):
         """
@@ -172,22 +171,14 @@ class MultiSurface(BaseSurface):
             if isinstance(surface, GriddedSurface):
                 return edges.append(surface.mesh)
             elif isinstance(surface, geo.surface.kite_fault.KiteSurface):
-                edge = []
-                mesh = surface.mesh
-                lons = mesh.lons
-                # We extract the top edge of the rupture from the
-                # corresponding 2D mesh.
-                # The calculation of indexes below is needed because we want
-                # on each 'profile' of the mesh the uppermost node that is
-                # finite (i.e. on the real grid)
-                for icol in range(lons.shape[1]):
-                    if numpy.all(numpy.isnan(lons[:, icol])):
-                        continue
-                    tmp = numpy.nonzero(numpy.isfinite(lons[:, icol]))[0]
-                    irow = tmp.argmax(axis=0)
-                    edge.append([mesh.lons[irow, icol], mesh.lats[irow, icol],
-                                 mesh.depths[irow, icol]])
-                edges.append(numpy.array(edge))
+                # The downsample_trace function will be replaced by a more
+                # appropriate function in a following PR
+                lo, la = surface.get_tor()
+                dep = numpy.tile(numpy.array([[0], [0.5]]), (1, len(lo)))
+                mesh = RectangularMesh(numpy.tile(lo, (2, 1)),
+                                       numpy.tile(la, (2, 1)),
+                                       dep)
+                edges.append(downsample_trace(mesh, tol))
             elif isinstance(surface, PlanarSurface):
                 # Top edge determined from two end points
                 edge = []
@@ -200,7 +191,7 @@ class MultiSurface(BaseSurface):
                 # overall size
                 edges.append(downsample_trace(surface.mesh, tol))
             else:
-                raise ValueError("Surface %s not recognised" % str(surface))
+                raise ValueError(f"Surface {str(surface)} not recognised")
         return edges
 
     def get_min_distance(self, mesh):
@@ -311,15 +302,11 @@ class MultiSurface(BaseSurface):
         formula for weighted mean is used.
         """
         areas = self._get_areas()
-        dips = numpy.array([numpy.mean(surf.get_dip()) for surf in
-                            self.surfaces])
-
-        ok = numpy.logical_and(numpy.isfinite(dips), numpy.isfinite(areas))
+        dips = numpy.array([surf.get_dip() for surf in self.surfaces])
+        ok = numpy.logical_and(numpy.isfinite(dips), numpy.isfinite(areas))[0]
         dips = dips[ok]
         areas = areas[ok]
-
         dip = numpy.sum(areas * dips) / numpy.sum(areas)
-        assert numpy.isfinite(dip).all()
         return dip
 
     def get_width(self):
@@ -329,7 +316,6 @@ class MultiSurface(BaseSurface):
         """
         areas = self._get_areas()
         widths = numpy.array([surf.get_width() for surf in self.surfaces])
-
         return numpy.sum(areas * widths) / numpy.sum(areas)
 
     def get_area(self):
@@ -389,13 +375,14 @@ class MultiSurface(BaseSurface):
         return numpy.array(self.surfaces)[idx][0].get_middle_point()
 
     def get_surface_boundaries(self):
-        lons = []
-        lats = []
-        for surf in self.surfaces:
-            lons_surf, lats_surf = surf.get_surface_boundaries()
-            lons.extend(lons_surf)
-            lats.extend(lats_surf)
-        return lons, lats
+        los, las = self.surfaces[0].get_surface_boundaries()
+        poly = Polygon((lo, la) for lo, la in zip(los, las))
+        for i in range(1, len(self.surfaces)):
+            los, las = self.surfaces[i].get_surface_boundaries()
+            polyt = Polygon([(lo, la) for lo, la in zip(los, las)])
+            poly = poly.union(polyt)
+        coo = numpy.array([[lo, la] for lo, la in list(poly.exterior.coords)])
+        return coo[:, 0], coo[:, 1]
 
     def get_surface_boundaries_3d(self):
         lons = []
@@ -428,12 +415,17 @@ class MultiSurface(BaseSurface):
         """
         # Get projection space for cartesian projection
         edge_sets = numpy.vstack(self.edge_set)
+        idx = numpy.isfinite(edge_sets[:, 0])
+        edge_sets = edge_sets[idx, :]
+        # Get bounding box
         west, east, north, south = utils.get_spherical_bounding_box(
             edge_sets[:, 0],
             edge_sets[:, 1])
         self.proj = utils.OrthographicProjection(west, east, north, south)
 
         for edges in self.edge_set:
+            idx = numpy.isfinite(edges[:, 0])
+            edges = edges[idx, :]
             # Project edges into cartesian space
             px, py = self.proj(edges[:, 0], edges[:, 1])
             # Store the two end-points of the trace
@@ -575,7 +567,6 @@ class MultiSurface(BaseSurface):
         # Loop over the traces
         for j, edges in enumerate(self.cartesian_edges):
 
-            # import pdb; pdb.set_trace()
             ok = numpy.isfinite(edges)
             if not ok.all():
                 edges = edges[ok]
