@@ -27,10 +27,103 @@ Module exports :class:`BindiEtAl2014Rjb`,
 import numpy as np
 from scipy.constants import g
 
+from openquake.baselib.general import CallableDict
 from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA
 
+CONSTS = {"Mref": 5.5,
+          "Mh": 6.75,
+          "Rref": 1.0,
+          "Vref": 800.0}
+
+
+def _get_distance_scaling_term(C, rval, mag):
+    """
+    Returns the distance scaling term of the GMPE described in equation 2
+    """
+    r_adj = np.sqrt(rval ** 2.0 + C["h"] ** 2.0)
+    return (
+        (C["c1"] + C["c2"] * (mag - CONSTS["Mref"])) *
+        np.log10(r_adj / CONSTS["Rref"]) -
+        (C["c3"] * (r_adj - CONSTS["Rref"])))
+
+
+def _get_magnitude_scaling_term(C, mag):
+    """
+    Returns the magnitude scaling term of the GMPE described in
+    equation 3
+    """
+    dmag = mag - CONSTS["Mh"]
+    if mag < CONSTS["Mh"]:
+        return C["e1"] + (C["b1"] * dmag) + (C["b2"] * (dmag ** 2.0))
+    else:
+        return C["e1"] + (C["b3"] * dmag)
+
+
+def _get_mean(kind, sof, C, ctx, dists):
+    """
+    Returns the mean ground motion
+    """
+    sof_term = _get_style_of_faulting_term(C, ctx) if sof else 0.
+    return (_get_magnitude_scaling_term(C, ctx.mag) +
+            _get_distance_scaling_term(C, dists, ctx.mag) +
+            _get_site_amplification_term(kind, C, ctx.vs30) + sof_term)
+
+
+_get_site_amplification_term = CallableDict()
+
+
+@_get_site_amplification_term.add("base")
+def _get_site_amplification_term_1(kind, C, vs30):
+    """
+    Returns the site amplification term for the case in which Vs30
+    is used directly
+    """
+    return C["gamma"] * np.log10(vs30 / CONSTS["Vref"])
+
+
+@_get_site_amplification_term.add("EC8")
+def _get_site_amplification_term_2(kind, C, vs30):
+    """
+    Returns the site amplification given Eurocode 8 site classification
+    """
+    f_s = np.zeros_like(vs30)
+    # Site class B
+    idx = np.logical_and(vs30 < 800.0, vs30 >= 360.0)
+    f_s[idx] = C["eB"]
+    # Site Class C
+    idx = np.logical_and(vs30 < 360.0, vs30 >= 180.0)
+    f_s[idx] = C["eC"]
+    # Site Class D
+    idx = vs30 < 180.0
+    f_s[idx] = C["eD"]
+    return f_s
+
+
+def _get_style_of_faulting_term(C, ctx):
+    """
+    Returns the style-of-faulting term.
+    Fault type (Strike-slip, Normal, Thrust/reverse) is
+    derived from rake angle.
+    Rakes angles within 30 of horizontal are strike-slip,
+    angles from 30 to 150 are reverse, and angles from
+    -30 to -150 are normal.
+    Note that the 'Unspecified' case is not considered in this class
+    as rake is required as an input variable
+    """
+    SS, NS, RS = 0.0, 0.0, 0.0
+    if np.abs(ctx.rake) <= 30.0 or (180.0 - np.abs(ctx.rake)) <= 30.0:
+        # strike-slip
+        SS = 1.0
+    elif ctx.rake > 30.0 and ctx.rake < 150.0:
+        # reverse
+        RS = 1.0
+    else:
+        # normal
+        NS = 1.0
+    return (C["sofN"] * NS) + (C["sofR"] * RS) + (C["sofS"] * SS)
+
 
 class BindiEtAl2014Rjb(GMPE):
     """
@@ -53,17 +146,15 @@ class BindiEtAl2014Rjb(GMPE):
     coefficients tables were taken from the Electronic Supplementary
     material of the original paper, which are indicated as being unaffected.
     """
+    kind = "base"
+
     #: Supported tectonic region type is 'active shallow crust'
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.ACTIVE_SHALLOW_CRUST
 
     #: Set of :mod:`intensity measure types <openquake.hazardlib.imt>`
     #: this GSIM can calculate. A set should contain classes from module
     #: :mod:`openquake.hazardlib.imt`.
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-        PGV,
-        SA
-    ])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, PGV, SA}
 
     #: Supported intensity measure component is the geometric mean of two
     #: horizontal components
@@ -71,11 +162,8 @@ class BindiEtAl2014Rjb(GMPE):
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     #: Required site parameter is only Vs30
     REQUIRES_SITES_PARAMETERS = {'vs30'}
@@ -86,107 +174,35 @@ class BindiEtAl2014Rjb(GMPE):
     #: Required distance measure is Rjb (eq. 1).
     REQUIRES_DISTANCES = {'rjb'}
 
+    sof = True
+
     def __init__(self, adjustment_factor=1.0, **kwargs):
         super().__init__(adjustment_factor=adjustment_factor, **kwargs)
         self.adjustment_factor = np.log(adjustment_factor)
+        [self.dist_type] = self.REQUIRES_DISTANCES
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        # extracting dictionary of coefficients specific to required
-        # intensity measure type.
+        dists = getattr(ctx, self.dist_type)
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            imean = _get_mean(self.kind, self.sof, C, ctx, dists)
+            if imt.string.startswith(('PGA', 'SA')):
+                # Convert units to g,
+                # but only for PGA and SA (not PGV)
+                mean[m] = np.log((10.0 ** (imean - 2.0)) / g)
+            else:
+                # PGV
+                mean[m] = np.log(10.0 ** imean)
 
-        C = self.COEFFS[imt]
-        imean = self._get_mean(C, rup, dists, sites)
-        if imt.name in "SA PGA":
-            # Convert units to g,
-            # but only for PGA and SA (not PGV):
-            mean = np.log((10.0 ** (imean - 2.0)) / g)
-        else:
-            # PGV:
-            mean = np.log(10.0 ** imean)
-
-        istddevs = self._get_stddevs(C, stddev_types, len(sites.vs30))
-        stddevs = np.log(10.0 ** np.array(istddevs))
-        return mean + self.adjustment_factor, stddevs
-
-    def _get_mean(self, C, rup, dists, sites):
-        """
-        Returns the mean ground motion
-        """
-        return (self._get_magnitude_scaling_term(C, rup.mag) +
-                self._get_distance_scaling_term(C, dists.rjb, rup.mag) +
-                self._get_style_of_faulting_term(C, rup) +
-                self._get_site_amplification_term(C, sites.vs30))
-
-    def _get_magnitude_scaling_term(self, C, mag):
-        """
-        Returns the magnitude scaling term of the GMPE described in
-        equation 3
-        """
-        dmag = mag - self.CONSTS["Mh"]
-        if mag < self.CONSTS["Mh"]:
-            return C["e1"] + (C["b1"] * dmag) + (C["b2"] * (dmag ** 2.0))
-        else:
-            return C["e1"] + (C["b3"] * dmag)
-
-    def _get_distance_scaling_term(self, C, rval, mag):
-        """
-        Returns the distance scaling term of the GMPE described in equation 2
-        """
-        r_adj = np.sqrt(rval ** 2.0 + C["h"] ** 2.0)
-        return (
-            (C["c1"] + C["c2"] * (mag - self.CONSTS["Mref"])) *
-            np.log10(r_adj / self.CONSTS["Rref"]) -
-            (C["c3"] * (r_adj - self.CONSTS["Rref"])))
-
-    def _get_style_of_faulting_term(self, C, rup):
-        """
-        Returns the style-of-faulting term.
-        Fault type (Strike-slip, Normal, Thrust/reverse) is
-        derived from rake angle.
-        Rakes angles within 30 of horizontal are strike-slip,
-        angles from 30 to 150 are reverse, and angles from
-        -30 to -150 are normal.
-        Note that the 'Unspecified' case is not considered in this class
-        as rake is required as an input variable
-        """
-        SS, NS, RS = 0.0, 0.0, 0.0
-        if np.abs(rup.rake) <= 30.0 or (180.0 - np.abs(rup.rake)) <= 30.0:
-            # strike-slip
-            SS = 1.0
-        elif rup.rake > 30.0 and rup.rake < 150.0:
-            # reverse
-            RS = 1.0
-        else:
-            # normal
-            NS = 1.0
-        return (C["sofN"] * NS) + (C["sofR"] * RS) + (C["sofS"] * SS)
-
-    def _get_site_amplification_term(self, C, vs30):
-        """
-        Returns the site amplification term for the case in which Vs30
-        is used directly
-        """
-        return C["gamma"] * np.log10(vs30 / self.CONSTS["Vref"])
-
-    def _get_stddevs(self, C, stddev_types, num_sites):
-        """
-        Return standard deviations as defined in table 2.
-        """
-        stddevs = []
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                stddevs.append(C['sigma'] + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(C['phi'] + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(C['tau'] + np.zeros(num_sites))
-        return stddevs
+            mean[m] += self.adjustment_factor
+            sig[m] = np.log(10.0 ** C['sigma'])
+            tau[m] = np.log(10.0 ** C['tau'])
+            phi[m] = np.log(10.0 ** C['phi'])
 
     #: Coefficients from Table 2
 
@@ -219,33 +235,13 @@ class BindiEtAl2014Rjb(GMPE):
     3.00   2.253990000   -0.940373000   0.227241000   5.741730000   0.000000000    0.385526000   -0.111445000   0.000000000   -0.732072000    0.022989300   -0.020662000   -0.002327150   0.176546000   0.314165000   0.207247000   0.360373000
     """)
 
-    CONSTS = {"Mref": 5.5,
-              "Mh": 6.75,
-              "Rref": 1.0,
-              "Vref": 800.0}
-
 
 class BindiEtAl2014RjbEC8(BindiEtAl2014Rjb):
     """
     Implements the Bindi et al (2014) GMPE for the case where Joyner-Boore
     distance is specified but Eurocode 8 Site classification is used.
     """
-
-    def _get_site_amplification_term(self, C, vs30):
-        """
-        Returns the site amplification given Eurocode 8 site classification
-        """
-        f_s = np.zeros_like(vs30)
-        # Site class B
-        idx = np.logical_and(vs30 < 800.0, vs30 >= 360.0)
-        f_s[idx] = C["eB"]
-        # Site Class C
-        idx = np.logical_and(vs30 < 360.0, vs30 >= 180.0)
-        f_s[idx] = C["eC"]
-        # Site Class D
-        idx = vs30 < 180.0
-        f_s[idx] = C["eD"]
-        return f_s
+    kind = "EC8"
 
     #: Coefficients from Table 1
     COEFFS = CoeffsTable(sa_damping=5, table="""
@@ -286,15 +282,7 @@ class BindiEtAl2014RjbEC8NoSOF(BindiEtAl2014RjbEC8):
     """
     #: Required rupture parameters are magnitude
     REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-
-    def _get_mean(self, C, rup, dists, sites):
-        """
-        Returns the mean value of ground motion - noting that in this case
-        the style-of-faulting term is neglected
-        """
-        return (self._get_magnitude_scaling_term(C, rup.mag) +
-                self._get_distance_scaling_term(C, dists.rjb, rup.mag) +
-                self._get_site_amplification_term(C, sites.vs30))
+    sof = False
 
 
 class BindiEtAl2014Rhyp(BindiEtAl2014Rjb):
@@ -303,21 +291,10 @@ class BindiEtAl2014Rhyp(BindiEtAl2014Rjb):
     distance is preferred, style-of-faulting is specfieid and for which the
     site amplification is dependent directly on Vs30
     """
-
     #: Required distance measure is Rhypo (eq. 1).
-    REQUIRES_DISTANCES = set(('rhypo', ))
-
-    def _get_mean(self, C, rup, dists, sites):
-        """
-        Returns the mean value of ground motion
-        """
-        return (self._get_magnitude_scaling_term(C, rup.mag) +
-                self._get_distance_scaling_term(C, dists.rhypo, rup.mag) +
-                self._get_style_of_faulting_term(C, rup) +
-                self._get_site_amplification_term(C, sites.vs30))
+    REQUIRES_DISTANCES = {'rhypo'}
 
     #: Coefficients from Table 4
-
     COEFFS = CoeffsTable(sa_damping=5, table="""
     imt             e1             c1             c2             h             c3            b1             b2            b3          gamma           sofN           sofR           sofS           tau           phi        phis2s         sigma
     pgv    3.242490000   -1.575560000    0.079177400   4.389180000   0.0000000000   0.472433000   -0.072548400   0.436952000   -0.508833000   -0.015719500    0.071385900   -0.055666000   0.193206000   0.295126000   0.178867000   0.352744000
@@ -355,16 +332,7 @@ class BindiEtAl2014RhypEC8(BindiEtAl2014RjbEC8):
     is characterised according to the Eurocode 8 site class
     """
     #: Required distance measure is Rhypo
-    REQUIRES_DISTANCES = set(('rhypo', ))
-
-    def _get_mean(self, C, rup, dists, sites):
-        """
-        Returns the mean value of ground motion
-        """
-        return (self._get_magnitude_scaling_term(C, rup.mag) +
-                self._get_distance_scaling_term(C, dists.rhypo, rup.mag) +
-                self._get_style_of_faulting_term(C, rup) +
-                self._get_site_amplification_term(C, sites.vs30))
+    REQUIRES_DISTANCES = {'rhypo'}
 
     #: Coefficients from Table 3
     COEFFS = CoeffsTable(sa_damping=5, table="""
@@ -406,12 +374,4 @@ class BindiEtAl2014RhypEC8NoSOF(BindiEtAl2014RhypEC8):
     """
     #: Required rupture parameters are magnitude
     REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-
-    def _get_mean(self, C, rup, dists, sites):
-        """
-        Returns the mean value of ground motion - noting that in this case
-        the style-of-faulting term is neglected
-        """
-        return (self._get_magnitude_scaling_term(C, rup.mag) +
-                self._get_distance_scaling_term(C, dists.rhypo, rup.mag) +
-                self._get_site_amplification_term(C, sites.vs30))
+    sof = False
