@@ -6,39 +6,74 @@ Module exports :class:`NBCC2015_AA13`
 import io
 import os
 import numpy as np
-from openquake.hazardlib.gsim.gmpe_table import GMPETable
+from openquake.hazardlib.gsim.gmpe_table import (
+    GMPETable, _return_tables, _get_mean, _get_stddevs)
 from openquake.hazardlib.gsim.base import CoeffsTable
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA
-from openquake.hazardlib.gsim.base import gsim_aliases
+from openquake.hazardlib.gsim.base import add_alias
+from openquake.hazardlib.gsim.atkinson_boore_2006 import (
+    _get_site_amplification_non_linear, _get_site_amplification_linear)
 from openquake.hazardlib.gsim.boore_atkinson_2008 import BooreAtkinson2008
 
 dirname = os.path.dirname(__file__)
 BASE_PATH_AA13 = os.path.join(dirname, 'nbcc2015_tables')
+BA08 = BooreAtkinson2008
 
-# populating `gsim_aliases` so that the engine can associate a string
-# to a specific gsim; for instance the string "NBCC2015_AA13_offshore_high"
-# is associated to the gsim (in TOML representation)
-# [NBCC2015_AA13]
-# REQUIRES_DISTANCES = ["rhypo"]
-# DEFINED_FOR_TECTONIC_REGION_TYPE = "Offshore"
-# gmpe_table = "Woffshore_high_clC.hdf5"
-arguments = [
-    ['stablecrust', 'rhypo', 'Stable Crust', 'ENA_%s_clC'],
-    ['activecrust', 'rhypo', 'Active Crust', 'Wcrust_%s_clC'],
-    ['activecrustFRjb', 'rjb', 'Active Crust Fault', 'WcrustFRjb_%s_clC'],
-    ['inslab30', 'rhypo', 'Subduction Inslab 30', 'WinslabD30_%s_clC'],
-    ['inslab50', 'rhypo', 'Subduction Inslab 50', 'WinslabD50_%s_clC'],
-    ['interface', 'rrup', 'Subduction Interface', 'WinterfaceCombo_%sclC'],
-    ['offshore', 'rhypo', 'Offshore', 'Woffshore_%s_clC']]
-for key, dist, trt, hdf5 in arguments:
-    for kind in ('low', 'med', 'high'):
-        name = f"NBCC2015_AA13_{key}_" + ("central" if kind == "med" else kind)
-        gsim_aliases[name] = f'''[NBCC2015_AA13]
-REQUIRES_DISTANCES = ["{dist}"]
-DEFINED_FOR_TECTONIC_REGION_TYPE = "{trt}"
-gmpe_table = "{hdf5}.hdf5"
-''' % kind
+
+def AB06_BA08(C, vs30, imt, PGA760):
+
+    F = np.zeros_like(vs30)
+
+    F[vs30 >= 760.] = 10**(np.interp(np.log10(vs30[vs30 >= 760.]),
+                                     np.log10([760.0, 2000.0]),
+                                     np.log10([1.0, C['c']])))
+    F[vs30 >= 760.] = 1./F[vs30 >= 760.]
+
+    C2 = BA08.COEFFS_SOIL_RESPONSE[imt]
+    nl = _get_site_amplification_non_linear(vs30[vs30 < 760.],
+                                            PGA760[vs30 < 760.],
+                                            C2)
+    lin = _get_site_amplification_linear(vs30[vs30 < 760.], C2)
+    F[vs30 < 760.] = np.exp(nl+lin)
+
+    return F
+
+
+def site_term(self, ctx, dists, imt):
+    """
+    Site term as used to calculate site coefficients for NBCC2015:
+
+    For Vs30 > 760 m/s use log-log interpolation of the 760-to-2000
+    factor of AA13.
+
+    For Vs30 < 760 m/s use the site term of Boore and Atkinson 2008.
+
+    Original site term is relative to Vs30 = 760m/s so needs to be made
+    relative to Vs30 = 450 m/s by dividing by the site term at Vs30 = 450.
+    Assume PGA_760 = 0.1g for Vs30 > 450 m/s. Also need to correct PGA at
+    site class C to 760 m/s. Cap PGA_450 at 0.1 - 0.5g.
+    """
+    imls_pga = _return_tables(self, ctx.mag, PGA(), "IMLs")
+    PGA450 = _get_mean(self.kind, self.distance_type, imls_pga, ctx, dists)
+    imls_SA02 = _return_tables(self, ctx.mag, SA(0.2), "IMLs")
+    SA02 = _get_mean(self.kind, self.distance_type, imls_SA02, ctx, dists)
+
+    PGA450[SA02 / PGA450 < 2.0] = PGA450[SA02 / PGA450 < 2.0] * 0.8
+
+    pgas = np.array([0.1, 0.2, 0.3, 0.4, 0.5])
+    pga_cors = np.array([0.083, 0.165, 0.248, 0.331, 0.414])
+    PGA760 = np.interp(PGA450, pgas, pga_cors)
+    PGA760_ref = np.copy(PGA760)
+    PGA760_ref[ctx.vs30 > 450.] = 0.1
+
+    vs30ref = np.zeros_like(ctx.vs30)
+    vs30ref += 450.0
+
+    C = self.COEFFS_2000_to_BC[imt]
+    site_term = (AB06_BA08(C, ctx.vs30, imt, PGA760_ref) /
+                 AB06_BA08(C, vs30ref, imt, PGA760_ref))
+    return np.log(site_term)
 
 
 class NBCC2015_AA13(GMPETable):
@@ -71,7 +106,6 @@ class NBCC2015_AA13(GMPETable):
     REQUIRES_SITES_PARAMETERS = {'vs30'}
     REQUIRES_DISTANCES = ""
     REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-    BA08 = BooreAtkinson2008()
 
     def __init__(self, **kwargs):
         # kwargs must contain the keys REQUIRES_DISTANCES,
@@ -89,79 +123,26 @@ class NBCC2015_AA13(GMPETable):
         self.DEFINED_FOR_TECTONIC_REGION_TYPE = kwargs[
             'DEFINED_FOR_TECTONIC_REGION_TYPE']
 
-    def get_mean_and_stddevs(self, sctx, rctx, dctx, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         Returns the mean and standard deviations
         """
-        # Return Distance Tables
-        imls = self._return_tables(rctx.mag, imt, "IMLs")
-        # Get distance vector for the given magnitude
-        idx = np.searchsorted(self.m_w, rctx.mag)
-        dists = self.distances[:, 0, idx - 1]
-        # Get mean and standard deviations
-        mean = np.log(self._get_mean(imls, dctx, dists))
-        stddevs = self._get_stddevs(dists, rctx.mag, dctx, imt, stddev_types)
-        amplification = self.site_term(sctx, rctx, dctx, dists, imt,
-                                       stddev_types)
-        mean += amplification
-        return mean, stddevs
-
-    def site_term(self, sctx, rctx, dctx, dists, imt, stddev_types):
-        """
-        Site term as used to calculate site coefficients for NBCC2015:
-
-        For Vs30 > 760 m/s use log-log interpolation of the 760-to-2000
-        factor of AA13.
-
-        For Vs30 < 760 m/s use the site term of Boore and Atkinson 2008.
-
-        Original site term is relative to Vs30 = 760m/s so needs to be made
-        relative to Vs30 = 450 m/s by dividing by the site term at Vs30 = 450.
-        Assume PGA_760 = 0.1g for Vs30 > 450 m/s. Also need to correct PGA at
-        site class C to 760 m/s. Cap PGA_450 at 0.1 - 0.5g.
-
-        """
-
-        imls_pga = self._return_tables(rctx.mag, PGA(), "IMLs")
-        PGA450 = self._get_mean(imls_pga, dctx, dists)
-        imls_SA02 = self._return_tables(rctx.mag, SA(0.2), "IMLs")
-        SA02 = self._get_mean(imls_SA02, dctx, dists)
-
-        PGA450[SA02 / PGA450 < 2.0] = PGA450[SA02 / PGA450 < 2.0] * 0.8
-
-        pgas = np.array([0.1, 0.2, 0.3, 0.4, 0.5])
-        pga_cors = np.array([0.083, 0.165, 0.248, 0.331, 0.414])
-        PGA760 = np.interp(PGA450, pgas, pga_cors)
-        PGA760_ref = np.copy(PGA760)
-        PGA760_ref[sctx.vs30 > 450.] = 0.1
-
-        vs30ref = np.zeros_like(sctx.vs30)
-        vs30ref += 450.0
-
-        site_term = (self.AB06_BA08(sctx.vs30, imt, PGA760_ref) /
-                     self.AB06_BA08(vs30ref, imt, PGA760_ref))
-
-        return np.log(site_term)
-
-    def AB06_BA08(self, vs30, imt, PGA760):
-
-        C = self.COEFFS_2000_to_BC[imt]
-
-        F = np.zeros_like(vs30)
-
-        F[vs30 >= 760.] = 10**(np.interp(np.log10(vs30[vs30 >= 760.]),
-                                         np.log10([760.0, 2000.0]),
-                                         np.log10([1.0, C['c']])))
-        F[vs30 >= 760.] = 1./F[vs30 >= 760.]
-
-        C2 = self.BA08.COEFFS_SOIL_RESPONSE[imt]
-        nl = self.BA08._get_site_amplification_non_linear(vs30[vs30 < 760.],
-                                                          PGA760[vs30 < 760.],
-                                                          C2)
-        lin = self.BA08._get_site_amplification_linear(vs30[vs30 < 760.], C2)
-        F[vs30 < 760.] = np.exp(nl+lin)
-
-        return F
+        stds = [sig, tau, phi]
+        stdis = [const.StdDev.idx[sdt] for sdt in
+                 self.DEFINED_FOR_STANDARD_DEVIATION_TYPES]
+        for m, imt in enumerate(imts):
+            # Return Distance Tables
+            imls = _return_tables(self, ctx.mag, imt, "IMLs")
+            # Get distance vector for the given magnitude
+            idx = np.searchsorted(self.m_w, ctx.mag)
+            dists = self.distances[:, 0, idx - 1]
+            # Get mean and standard deviations
+            mean[m] = np.log(_get_mean(
+                self.kind, self.distance_type, imls, ctx, dists)
+            ) + site_term(self, ctx, dists, imt)
+            stddevs = _get_stddevs(self, dists, ctx, imt, stdis)
+            for s in stdis:
+                stds[s][m] = stddevs[s]
 
     COEFFS_2000_to_BC = CoeffsTable(sa_damping=5, table="""\
     IMT     c
@@ -177,3 +158,27 @@ class NBCC2015_AA13(GMPETable):
     5.0 1.148
     10.0 1.072
     """)
+
+
+# populating `gsim_aliases` so that the engine can associate a string
+# to a specific gsim; for instance the string "NBCC2015_AA13_offshore_high"
+# is associated to the gsim (in TOML representation)
+# [NBCC2015_AA13]
+# REQUIRES_DISTANCES = ["rhypo"]
+# DEFINED_FOR_TECTONIC_REGION_TYPE = "Offshore"
+# gmpe_table = "Woffshore_high_clC.hdf5"
+arguments = [
+    ['stablecrust', 'rhypo', 'Stable Crust', 'ENA_%s_clC'],
+    ['activecrust', 'rhypo', 'Active Crust', 'Wcrust_%s_clC'],
+    ['activecrustFRjb', 'rjb', 'Active Crust Fault', 'WcrustFRjb_%s_clC'],
+    ['inslab30', 'rhypo', 'Subduction Inslab 30', 'WinslabD30_%s_clC'],
+    ['inslab50', 'rhypo', 'Subduction Inslab 50', 'WinslabD50_%s_clC'],
+    ['interface', 'rrup', 'Subduction Interface', 'WinterfaceCombo_%sclC'],
+    ['offshore', 'rhypo', 'Offshore', 'Woffshore_%s_clC']]
+for key, dist, trt, hdf5 in arguments:
+    for kind in ('low', 'med', 'high'):
+        name = f"NBCC2015_AA13_{key}_" + ("central" if kind == "med" else kind)
+        add_alias(name, NBCC2015_AA13,
+                  REQUIRES_DISTANCES=[dist],
+                  DEFINED_FOR_TECTONIC_REGION_TYPE=trt,
+                  gmpe_table=f"{hdf5}.hdf5" % kind)
