@@ -20,14 +20,11 @@
 Module exports :class:`MontalvaEtAl2016SInter`
                :class:`MontalvaEtAl2016SSlab`
 """
-import numpy as np
-
 from openquake.hazardlib.gsim.base import CoeffsTable
-from openquake.hazardlib.imt import PGA
-from openquake.hazardlib.gsim.abrahamson_2015 import (AbrahamsonEtAl2015SInter,
-                                                      AbrahamsonEtAl2015SSlab)
-from openquake.hazardlib.gsim.montalva_2017 import (MontalvaEtAl2017SInter,
-                                                    MontalvaEtAl2017SSlab)
+from openquake.hazardlib.gsim.abrahamson_2015 import (
+    AbrahamsonEtAl2015SInter, AbrahamsonEtAl2015SSlab)
+from openquake.hazardlib.gsim.montalva_2017 import (
+    MontalvaEtAl2017SInter, MontalvaEtAl2017SSlab)
 
 
 class MontalvaEtAl2016SInter(AbrahamsonEtAl2015SInter):
@@ -48,53 +45,7 @@ class MontalvaEtAl2016SInter(AbrahamsonEtAl2015SInter):
     hazard models using this implementation
     """
     superseded_by = MontalvaEtAl2017SInter
-    
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        """
-        See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
-        for spec of input and result values.
-        """
-        # extract dictionaries of coefficients specific to required
-        # intensity measure type and for PGA
-        C = self.COEFFS[imt]
-        C_PGA = self.COEFFS[PGA()]
-        dc1_pga = C_PGA["DC1"]
-        # compute median pga on rock (vs30=1000), needed for site response
-        # term calculation
-        pga1000 = np.exp(
-            self._compute_pga_rock(C_PGA, dc1_pga, sites, rup, dists))
-        mean = (self._compute_magnitude_term(C, C["DC1"], rup.mag) +
-                self._compute_distance_term(C, rup.mag, dists) +
-                self._compute_focal_depth_term(C, rup) +
-                self._compute_forearc_backarc_term(C, sites, dists) +
-                self._compute_site_response_term(C, sites, pga1000))
-        stddevs = self._get_stddevs(C, stddev_types, len(sites.vs30))
-        return mean, stddevs
-
-    def _compute_magnitude_term(self, C, dc1, mag):
-        """
-        Computes the magnitude scaling term given by equation (2)
-        """
-        base = C['theta1'] + (C['theta4'] * dc1)
-        dmag = self.CONSTS["C1"] + dc1
-        if mag > dmag:
-            f_mag = (C['theta5'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        else:
-            f_mag = (C['theta4'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        return base + f_mag
-
-    def _compute_distance_term(self, C, mag, dists):
-        """
-        Computes the distance scaling term, as contained within equation (1)
-        """
-        return (C['theta2'] + C['theta3'] * (mag - 7.8)) *\
-            np.log(dists.rrup + self.CONSTS['c4'] * np.exp((mag - 6.) *
-                   self.CONSTS['theta9'])) + (C['theta6'] * dists.rrup)
+    kind = "montalva16"
 
     COEFFS = CoeffsTable(sa_damping=5, table="""\
     imt              DC1     vlin       b        theta1         theta2        theta3         theta4         theta5         theta6    theta7  theta8       theta10        theta11        theta12        theta13        theta14  theta15 theta16           phi           tau         sigma       phi_s2s
@@ -124,15 +75,6 @@ class MontalvaEtAl2016SInter(AbrahamsonEtAl2015SInter):
     10.00   -0.200000000    400.0   0.000   3.266979586   -1.707902316   0.068210457   -0.967817098    0.253077379    0.004736644   -0.4828    0.30   7.382937906    0.000738462   -0.423369635   -0.347713953   -1.409670235   0.3000    0.00   0.466924628   0.376696614   0.599932452   0.300789811
     """)
 
-    CONSTS = {
-        # Period-Independent Coefficients (Table 2)
-        'n': 1.18,
-        'c': 1.88,
-        'c4': 10.0,
-        'C1': 7.8,
-        'theta9': 0.4
-        }
-
 
 class MontalvaEtAl2016SSlab(AbrahamsonEtAl2015SSlab):
     """
@@ -148,55 +90,7 @@ class MontalvaEtAl2016SSlab(AbrahamsonEtAl2015SSlab):
     """
     superseded_by = MontalvaEtAl2017SSlab
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        """
-        See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
-        for spec of input and result values.
-        """
-        # extract dictionaries of coefficients specific to required
-        # intensity measure type and for PGA
-        C = self.COEFFS[imt]
-        # For inslab GMPEs the correction term is fixed at -0.3
-        dc1 = -0.3
-        C_PGA = self.COEFFS[PGA()]
-        # compute median pga on rock (vs30=1000), needed for site response
-        # term calculation
-        pga1000 = np.exp(
-            self._compute_pga_rock(C_PGA, dc1, sites, rup, dists))
-        mean = (self._compute_magnitude_term(C, dc1, rup.mag) +
-                self._compute_distance_term(C, rup.mag, dists) +
-                self._compute_focal_depth_term(C, rup) +
-                self._compute_forearc_backarc_term(C, sites, dists) +
-                self._compute_site_response_term(C, sites, pga1000))
-        stddevs = self._get_stddevs(C, stddev_types, len(sites.vs30))
-        return mean, stddevs
-
-    def _compute_magnitude_term(self, C, dc1, mag):
-        """
-        Computes the magnitude scaling term given by equation (2)
-        corrected by a local adjustment factor
-        """
-        base = C['theta1'] + (C['theta4'] * dc1)
-        dmag = self.CONSTS["C1"] + dc1
-        if mag > dmag:
-            f_mag = (C['theta5'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        else:
-            f_mag = (C['theta4'] * (mag - dmag)) +\
-                C['theta13'] * ((10. - mag) ** 2.)
-
-        return base + f_mag
-
-    def _compute_distance_term(self, C, mag, dists):
-        """
-        Computes the distance scaling term, as contained within equation (1b)
-        """
-        return ((C['theta2'] + C['theta14'] + C['theta3'] *
-                (mag - 7.8)) * np.log(dists.rhypo + self.CONSTS['c4'] *
-                np.exp((mag - 6.) * self.CONSTS['theta9'])) +
-                (C['theta6'] * dists.rhypo)) + C["theta10"]
+    kind = "montalva16"
 
     COEFFS = CoeffsTable(sa_damping=5, table="""\
     imt              DC1    vlin        b        theta1         theta2        theta3         theta4         theta5         theta6    theta7  theta8       theta10        theta11        theta12        theta13        theta14  theta15 theta16           phi           tau         sigma       phi_s2s
