@@ -30,6 +30,83 @@ from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
 from openquake.hazardlib.imt import PGA, SA
 
 
+def get_stddevs(C):
+    """
+    Return standard deviations.
+    """
+    phi_tot = math.sqrt(C['phiss'] ** 2 + C['phis2s'] ** 2)
+    return [np.sqrt(C["tau"] ** 2 + phi_tot ** 2),
+            C["tau"],
+            phi_tot]
+
+
+def _basin_term(region, C, vs30, z1pt0):
+    """
+    Basin term [16].
+    """
+    if region == 'glb':
+        return 0
+
+    if region == 'tw':
+        ez_1 = np.exp(-3.73 / 2 * np.log((vs30 ** 2 + 290.53 ** 2)
+                                         / (1750 ** 2 + 290.53 ** 2)))
+        phi6 = 300
+    elif region in ['ca', 'jp']:
+        ez_1 = np.exp(-5.23 / 2 * np.log((vs30 ** 2 + 412.39 ** 2)
+                                         / (1360 ** 2 + 412.39 ** 2)))
+        if region == 'ca':
+            phi6 = 300
+        else:
+            phi6 = 800
+
+    d_z1 = z1pt0 - ez_1
+    return np.where(
+        z1pt0 < 0, 0, C['phi5' + region] * (1 - np.exp(-d_z1 / phi6)))
+
+
+def _distance_attenuation(s, region, aftershocks, C, mag, rrup, ztor):
+    """
+    Distance scaling and attenuation term [8, 9, 10].
+    """
+    del_c5 = C['dp'] * max(ztor / 50 - 20 / 50, 0) \
+        * (ztor > 20) * (mag < 7.0)
+    cns = (C['c5'] + del_c5) * math.cosh(C['c6'] * max(mag - C['c_hm'], 0))
+
+    f8 = s['c4'] * np.log(rrup + cns)
+    f9 = (s['c4_a'] - s['c4']) * np.log(np.sqrt(rrup ** 2
+                                                + s['c_rb'] ** 2))
+    f10 = (C['c_g1' + region] + aftershocks * C['dc_g1as']
+           + C['c_g2'] / (math.cosh(max(mag - C['c_g3'], 0)))) * rrup
+
+    return f8 + f9 + f10
+
+
+def _fsof_ztor(C, mag, rake, ztor):
+    """
+    Factors requiring type of fault.
+    """
+    f234 = 0
+    if 30 <= rake <= 150:
+        e_ztor = (max(3.5384 - 2.60 * max(mag - 5.8530, 0), 0)) ** 2
+        # reverse fault [2]
+        f234 += C['c1_a'] \
+            + C['c1_c'] / math.cosh(2 * max(mag - 4.5, 0))
+    else:
+        e_ztor = max(2.7482 - 1.7639 * max(mag - 5.5210, 0), 0) ** 2
+        if -120 <= rake <= -60:
+            # normal fault [3]
+            f234 += C['c1_b'] \
+                + C['c1_d'] / math.cosh(2 * max(mag - 4.5, 0))
+
+    ztor = np.where(ztor < 0, e_ztor, ztor)
+    delta_ztor = ztor - e_ztor
+    # [4]
+    f234 += (C['c7'] + C['c7_b'] / math.cosh(2 * max(mag - 4.5, 0))) \
+        * delta_ztor
+
+    return f234, ztor
+
+
 class PhungEtAl2020Asc(GMPE):
     """
     Implements Phung et al. (2020) for crustal.
@@ -43,11 +120,8 @@ class PhungEtAl2020Asc(GMPE):
 
     DEFINED_FOR_REFERENCE_VELOCITY = 1130
 
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     # rx for hanging wall
     REQUIRES_DISTANCES = {'rjb', 'rrup', 'rx'}
@@ -68,135 +142,52 @@ class PhungEtAl2020Asc(GMPE):
         # direct point parameter for directivity effect
         self.d_dpp = d_dpp
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
-        # extract dictionaries of coefficients specific to IM type
-        C = self.COEFFS[imt]
-
-        lnmed, ztor = self._fsof_ztor(C, rup.mag, rup.rake, rup.ztor)
-        # main shock [1]
-        lnmed += C['c1']
-        # dip term [5]
-        lnmed += (C['c11'] + C['c11_b']
-                  / math.cosh(2 * max(rup.mag - 4.5, 0))) \
-            * (math.cos(math.radians(rup.dip)) ** 2)
-        # hanging wall term [12]
-        lnmed += (dists.rx >= 0) * C['c9'] * math.cos(math.radians(rup.dip)) \
-            * (C['c9_a'] + (1 - C['c9_a']) * np.tanh(dists.rx / C['c9_b'])) \
-            * (1 - np.sqrt(dists.rjb ** 2 + ztor ** 2) / (dists.rrup + 1))
-        # directivity [11]
-        lnmed += C['c8'] \
-            * np.maximum(1 - np.maximum(dists.rrup - 40, 0) / 30, 0) \
-            * min(max(rup.mag - 5.5, 0) / 0.8, 1) \
-            * math.exp(self.CONSTANTS['c8_a'] * (rup.mag - C['c8_b']) ** 2) \
-            * self.d_dpp
-        # fmag [6, 7]
-        lnmed += self.CONSTANTS['c2'] * (rup.mag - 6)
-        lnmed += (self.CONSTANTS['c2'] - C['c3']) / C['c_n'] \
-            * math.log(1 + math.exp(C['c_n'] * (C['c_m'] - rup.mag)))
-        lnmed += self._distance_attenuation(C, rup.mag, dists.rrup, ztor)
-
-        sa1130 = np.exp(lnmed)
-        # site response [14, 15]
-        lnmed += C['phi1' + self.region] \
-            * np.minimum(np.log(sites.vs30 / 1130), 0)
-        lnmed += C['phi2'] \
-            * (np.exp(C['phi3'] * (np.minimum(sites.vs30, 1130) - 360))
-               - math.exp(C['phi3'] * (1130 - 360))) \
-            * np.log((sa1130 + C['phi4']) / C['phi4'])
-        # basin term [16]
-        lnmed += self._basin_term(C, sites.vs30, sites.z1pt0)
-
-        stddevs = self.get_stddevs(C, stddev_types)
-
-        return lnmed, stddevs
-
-    def get_stddevs(self, C, stddev_types):
-        """
-        Return standard deviations.
-        """
-        phi_tot = math.sqrt(C['phiss'] ** 2 + C['phis2s'] ** 2)
-
-        stddevs = []
-        for stddev in stddev_types:
-            assert stddev in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev == const.StdDev.TOTAL:
-                stddevs.append(np.sqrt(C["tau"] ** 2 + phi_tot ** 2))
-            elif stddev == const.StdDev.INTER_EVENT:
-                stddevs.append(C["tau"])
-            elif stddev == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi_tot)
-
-        return stddevs
-
-    def _basin_term(self, C, vs30, z1pt0):
-        """
-        Basin term [16].
-        """
-        if self.region == 'glb':
-            return 0
-
-        if self.region == 'tw':
-            ez_1 = np.exp(-3.73 / 2 * np.log((vs30 ** 2 + 290.53 ** 2)
-                                             / (1750 ** 2 + 290.53 ** 2)))
-            phi6 = 300
-        elif self.region in ['ca', 'jp']:
-            ez_1 = np.exp(-5.23 / 2 * np.log((vs30 ** 2 + 412.39 ** 2)
-                                             / (1360 ** 2 + 412.39 ** 2)))
-            if self.region == 'ca':
-                phi6 = 300
-            else:
-                phi6 = 800
-
-        d_z1 = z1pt0 - ez_1
-        return np.where(z1pt0 < 0, 0,
-                        C['phi5' + self.region] * (1 - np.exp(-d_z1 / phi6)))
-
-    def _distance_attenuation(self, C, mag, rrup, ztor):
-        """
-        Distance scaling and attenuation term [8, 9, 10].
-        """
-        del_c5 = C['dp'] * max(ztor / 50 - 20 / 50, 0) \
-            * (ztor > 20) * (mag < 7.0)
-        cns = (C['c5'] + del_c5) * math.cosh(C['c6'] * max(mag - C['c_hm'], 0))
         s = self.CONSTANTS
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
 
-        f8 = s['c4'] * np.log(rrup + cns)
-        f9 = (s['c4_a'] - s['c4']) * np.log(np.sqrt(rrup ** 2
-                                                    + s['c_rb'] ** 2))
-        f10 = (C['c_g1' + self.region] + self.aftershocks * C['dc_g1as']
-               + C['c_g2'] / (math.cosh(max(mag - C['c_g3'], 0)))) * rrup
+            lnmed, ztor = _fsof_ztor(C, ctx.mag, ctx.rake, ctx.ztor)
+            # main shock [1]
+            lnmed += C['c1']
+            # dip term [5]
+            lnmed += (C['c11'] + C['c11_b']
+                      / math.cosh(2 * max(ctx.mag - 4.5, 0))) \
+                * (math.cos(math.radians(ctx.dip)) ** 2)
+            # hanging wall term [12]
+            lnmed += (ctx.rx >= 0) * C['c9'] * math.cos(math.radians(ctx.dip))\
+                * (C['c9_a'] + (1 - C['c9_a']) * np.tanh(ctx.rx / C['c9_b']))\
+                * (1 - np.sqrt(ctx.rjb ** 2 + ztor ** 2) / (ctx.rrup + 1))
+            # directivity [11]
+            lnmed += C['c8'] \
+                * np.maximum(1 - np.maximum(ctx.rrup - 40, 0) / 30, 0) \
+                * min(max(ctx.mag - 5.5, 0) / 0.8, 1) \
+                * math.exp(s['c8_a'] * (ctx.mag - C['c8_b']) ** 2) \
+                * self.d_dpp
+            # fmag [6, 7]
+            lnmed += s['c2'] * (ctx.mag - 6)
+            lnmed += (s['c2'] - C['c3']) / C['c_n'] \
+                * math.log(1 + math.exp(C['c_n'] * (C['c_m'] - ctx.mag)))
+            lnmed += _distance_attenuation(
+                s, self.region, self.aftershocks, C, ctx.mag, ctx.rrup, ztor)
 
-        return f8 + f9 + f10
-
-    def _fsof_ztor(self, C, mag, rake, ztor):
-        """
-        Factors requiring type of fault.
-        """
-        f234 = 0
-        if 30 <= rake <= 150:
-            e_ztor = (max(3.5384 - 2.60 * max(mag - 5.8530, 0), 0)) ** 2
-            # reverse fault [2]
-            f234 += C['c1_a'] \
-                + C['c1_c'] / math.cosh(2 * max(mag - 4.5, 0))
-        else:
-            e_ztor = max(2.7482 - 1.7639 * max(mag - 5.5210, 0), 0) ** 2
-            if -120 <= rake <= -60:
-                # normal fault [3]
-                f234 += C['c1_b'] \
-                    + C['c1_d'] / math.cosh(2 * max(mag - 4.5, 0))
-
-        ztor = np.where(ztor < 0, e_ztor, ztor)
-        delta_ztor = ztor - e_ztor
-        # [4]
-        f234 += (C['c7'] + C['c7_b'] / math.cosh(2 * max(mag - 4.5, 0))) \
-            * delta_ztor
-
-        return f234, ztor
+            sa1130 = np.exp(lnmed)
+            # site response [14, 15]
+            lnmed += C['phi1' + self.region] * np.minimum(
+                np.log(ctx.vs30 / 1130), 0)
+            lnmed += C['phi2'] * (np.exp(C['phi3'] * (
+                np.minimum(ctx.vs30, 1130) - 360)) - math.exp(
+                    C['phi3'] * (1130 - 360))) * np.log(
+                        (sa1130 + C['phi4']) / C['phi4'])
+            # basin term [16]
+            lnmed += _basin_term(self.region, C, ctx.vs30, ctx.z1pt0)
+            mean[m] = lnmed
+            sig[m], tau[m], phi[m] = get_stddevs(C)
 
     COEFFS = CoeffsTable(sa_damping=5, table="""\
      imt    c1     c1_a         c1_b        c1_c         c1_d        c3     c5     c6      c7           c7_b        c8     c8_b   c9     c9_a   c9_b    c11         c11_b        c12         c12_b        c_n         c_m      c_g2        c_g3        c_hm    dp           phi2    phi3     phi4      c_g1tw     phi1tw       dc_g1as      c_g1ca       phi1ca      c_g1jp        phi1jp       c_g1glb      phi1glb     phi5tw  phi5ca      phi5jp      tau         phiss  phis2s
@@ -231,6 +222,101 @@ class PhungEtAl2020Asc(GMPE):
                  'c_rb': 50}
 
 
+def _get_stddevs(region, C):
+    """
+    Return standard deviations.
+    """
+    phi_tot = np.sqrt(C['phiss4' + region] ** 2 + C['phis2s4' + region] ** 2)
+    return [np.sqrt(C["tau4" + region] ** 2 + phi_tot ** 2),
+            C["tau4" + region],
+            phi_tot]
+
+
+def _fmag(region, C, mag, pga=False):
+    """
+    Magnitude term.
+    """
+    a4 = C['a4']
+    if region == 'jptw':
+        a4 += C['a4_del']
+
+    if mag <= C['mref']:
+        return a4 * (mag - C['mref']) + C['a13'] * (10 - mag) ** 2
+    return C['a5'] * (mag - C['mref']) + C['a13'] * (10 - mag) ** 2
+
+
+def _fp(trt, region, s, C, mag, rrup, pga=False):
+    """
+    Path term for subduction interface.
+    """
+    a7, a14 = (C['a7'], C['a14']) if (
+        trt == const.TRT.SUBDUCTION_INTRASLAB) else (0, 0)
+    a1 = C['a1']
+    if region == 'jptw':
+        a1 += C['a1_del']
+    r = 'tw' if pga else region
+    return a1 + a7 + (C['a2'] + a14 + s['a3'] * (mag - 7.8)) \
+        * np.log(rrup + s['c4'] * math.exp(s['a9'] * (mag - 6))) \
+        + C['a6' + r] * rrup
+
+
+def _fsite(s, region, C, vs30, pga_1000):
+    """
+    Non-linear component.
+    """
+    result = C['a12' + region] * np.log(vs30 / C['vlin'])
+    idx = vs30 < C['vlin']
+
+    result[idx] += \
+        - C['b'] * np.log(pga_1000[idx] + s['c']) \
+        + C['b'] * np.log(pga_1000[idx] + s['c']
+                          * (vs30[idx] / C['vlin']) ** s['n'])
+    result[~idx] += C['b'] * s['n'] * np.log(vs30[~idx] / C['vlin'])
+
+    return result
+
+
+def _fz1pt0(region, C, vs30, z1pt0):
+    """
+    Basin depth term.
+    """
+    result = np.zeros_like(z1pt0)
+    idx = np.where(z1pt0 >= 0)
+
+    if region == 'tw':
+        ez_1 = np.exp(-3.96 / 2 * np.log((vs30 ** 2 + 352.7 ** 2)
+                                         / (1750 ** 2 + 352.7 ** 2)))
+    elif region == 'jptw':
+        ez_1 = np.exp(-5.23 / 2 * np.log((vs30 ** 2 + 412.39 ** 2)
+                                         / (1360 ** 2 + 412.39 ** 2)))
+
+    result[idx] = C['a8' + region] * np.minimum(np.log(z1pt0[idx] / ez_1), 1)
+    return result
+
+
+def _fztor(trt, C, ztor):
+    """
+    Ztor factor for subduction interface.
+    """
+    if trt == const.TRT.SUBDUCTION_INTRASLAB:
+        return C['a11'] * (min(ztor, 80) - 40)
+    return C['a10'] * (min(ztor, 40) - 20)
+
+
+def pga_rock(trt, region, s, C_PGA, mag, rrup, ztor, vs30):
+    """
+    PGA at Vs30 (as Taiwan region, C_PGA)
+    """
+    f_mag = _fmag(region, C_PGA, mag, pga=True)
+    f_ztor = _fztor(trt, C_PGA, ztor)
+    f_p = _fp(trt, region, s, C_PGA, mag, rrup, pga=True)
+    # site function
+    f_site = C_PGA['a12tw'] * np.log(vs30 / C_PGA['vlin']) \
+        + C_PGA['b'] * s['n'] * np.log(vs30 / C_PGA['vlin'])
+
+    return np.exp(f_mag + f_p + f_ztor + f_site)
+
+
 class PhungEtAl2020SInter(GMPE):
     """
     Implements Phung et al. (2020) for Subduction Interface.
@@ -238,17 +324,14 @@ class PhungEtAl2020SInter(GMPE):
 
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTERFACE
 
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([PGA, SA])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, SA}
 
     DEFINED_FOR_INTENSITY_MEASURE_COMPONENT = const.IMC.AVERAGE_HORIZONTAL
 
     DEFINED_FOR_REFERENCE_VELOCITY = 1000
 
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     # rjb and rx not required for subduction
     REQUIRES_DISTANCES = {'rrup'}
@@ -265,135 +348,36 @@ class PhungEtAl2020SInter(GMPE):
         # 'jptw', 'tw' (Japan/Taiwan joined, Taiwan)
         self.region = region
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
+    def compute(self, ctx, imts, mean, sig, tau, phi):
         """
         See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
+        <.base.GroundShakingIntensityModel.compute>`
         for spec of input and result values.
         """
         # extract dictionaries of coefficients specific to IM type
-        C = self.COEFFS[imt]
+        trt = self.DEFINED_FOR_TECTONIC_REGION_TYPE
         C_PGA = self.COEFFS[PGA()]
-
-        f_mag = self._fmag(C, rup.mag)
-        f_ztor = self._fztor(C, rup.ztor)
-        # path term
-        f_p = self._fp(C, rup.mag, dists.rrup)
-        # PGA at rock with Vs30 = 1000 m/s
-        pga_1000 = self.pga_rock(C_PGA, rup.mag, dists.rrup, rup.ztor, 1000)
-        # site effect
-        vs30 = np.minimum(sites.vs30, 1000)
-        # non-linear component
-        f_site = self._fsite(C, vs30, pga_1000)
-        # basin depth term
-        f_z1pt0 = self._fz1pt0(C, vs30, sites.z1pt0)
-
-        # median total and stddev
-        sa = f_mag + f_p + f_ztor + f_site + f_z1pt0
-        stddev = self.get_stddevs(C, stddev_types)
-        return sa, stddev
-
-    def get_stddevs(self, C, stddev_types):
-        """
-        Return standard deviations.
-        """
-        phi_tot = math.sqrt(C['phiss4' + self.region] ** 2
-                            + C['phis2s4' + self.region] ** 2)
-
-        stddevs = []
-        for stddev in stddev_types:
-            assert stddev in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev == const.StdDev.TOTAL:
-                stddevs.append(np.sqrt(C["tau4" + self.region] ** 2
-                                       + phi_tot ** 2))
-            elif stddev == const.StdDev.INTER_EVENT:
-                stddevs.append(C["tau4" + self.region])
-            elif stddev == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi_tot)
-
-        return stddevs
-
-    def _fmag(self, C, mag, pga=False):
-        """
-        Magnitude term.
-        """
-        a4 = C['a4']
-        if self.region == 'jptw':
-            a4 += C['a4_del']
-
-        if mag <= C['mref']:
-            return a4 * (mag - C['mref']) + C['a13'] * (10 - mag) ** 2
-        return C['a5'] * (mag - C['mref']) + C['a13'] * (10 - mag) ** 2
-
-    def _fp(self, C, mag, rrup, pga=False):
-        """
-        Path term for subduction interface.
-        """
         s = self.CONSTANTS
-        a1 = C['a1']
-        if self.region == 'jptw':
-            a1 += C['a1_del']
-        if pga:
-            r = 'tw'
-        else:
-            r = self.region
-        return a1 + (C['a2'] + s['a3'] * (mag - 7.8)) \
-            * np.log(rrup + s['c4'] * math.exp(s['a9'] * (mag - 6))) \
-            + C['a6' + r] * rrup
+        for m, imt in enumerate(imts):
+            C = self.COEFFS[imt]
+            f_mag = _fmag(self.region, C, ctx.mag)
+            f_ztor = _fztor(trt, C, ctx.ztor)
+            # path term
+            f_p = _fp(trt, self.region, s, C, ctx.mag, ctx.rrup)
+            # PGA at rock with Vs30 = 1000 m/s
+            pga_1000 = pga_rock(
+                trt, self.region, self.CONSTANTS, C_PGA, ctx.mag,
+                ctx.rrup, ctx.ztor, 1000)
+            # site effect
+            vs30 = np.minimum(ctx.vs30, 1000)
+            # non-linear component
+            f_site = _fsite(s, self.region, C, vs30, pga_1000)
+            # basin depth term
+            f_z1pt0 = _fz1pt0(self.region, C, vs30, ctx.z1pt0)
 
-    def _fsite(self, C, vs30, pga_1000):
-        """
-        Non-linear component.
-        """
-        s = self.CONSTANTS
-        result = C['a12' + self.region] * np.log(vs30 / C['vlin'])
-        idx = vs30 < C['vlin']
-
-        result[idx] += \
-            - C['b'] * np.log(pga_1000[idx] + s['c']) \
-            + C['b'] * np.log(pga_1000[idx] + s['c']
-                              * (vs30[idx] / C['vlin']) ** s['n'])
-        result[~idx] += C['b'] * s['n'] * np.log(vs30[~idx] / C['vlin'])
-
-        return result
-
-    def _fz1pt0(self, C, vs30, z1pt0):
-        """
-        Basin depth term.
-        """
-        result = np.zeros_like(z1pt0)
-        idx = np.where(z1pt0 >= 0)
-
-        if self.region == 'tw':
-            ez_1 = np.exp(-3.96 / 2 * np.log((vs30 ** 2 + 352.7 ** 2)
-                                             / (1750 ** 2 + 352.7 ** 2)))
-        elif self.region == 'jptw':
-            ez_1 = np.exp(-5.23 / 2 * np.log((vs30 ** 2 + 412.39 ** 2)
-                                             / (1360 ** 2 + 412.39 ** 2)))
-
-        result[idx] = C['a8' + self.region] \
-            * np.minimum(np.log(z1pt0[idx] / ez_1), 1)
-        return result
-
-    def _fztor(self, C, ztor):
-        """
-        Ztor factor for subduction interface.
-        """
-        return C['a10'] * (min(ztor, 40) - 20)
-
-    def pga_rock(self, C_PGA, mag, rrup, ztor, vs30):
-        """
-        PGA at Vs30 (as Taiwan region, C_PGA)
-        """
-        s = self.CONSTANTS
-        f_mag = self._fmag(C_PGA, mag, pga=True)
-        f_ztor = self._fztor(C_PGA, ztor)
-        f_p = self._fp(C_PGA, mag, rrup, pga=True)
-        # site function
-        f_site = C_PGA['a12tw'] * np.log(vs30 / C_PGA['vlin']) \
-            + C_PGA['b'] * s['n'] * np.log(vs30 / C_PGA['vlin'])
-
-        return np.exp(f_mag + f_p + f_ztor + f_site)
+            # median total and stddev
+            mean[m] = f_mag + f_p + f_ztor + f_site + f_z1pt0
+            sig[m], tau[m], phi[m] = _get_stddevs(self.region, C)
 
     COEFFS = CoeffsTable(sa_damping=5, table="""\
     imt   a1      a1_del       a2          a4           a4_del      a5          a6jptw       a6tw         a7           a8jptw  a8tw    a10         a11          a12jptw a12tw   a13        a14          b     mref vlin   tau4jptw    phiss4jptw  phis2s4jptw tau4tw      phiss4tw phis2s4tw
@@ -427,27 +411,4 @@ class PhungEtAl2020SSlab(PhungEtAl2020SInter):
     """
     Implements Phung et al. (2020) for Subduction Intraslab.
     """
-
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.SUBDUCTION_INTRASLAB
-
-    def _fp(self, C, mag, rrup, pga=False):
-        """
-        Path term for subduction intraslab.
-        """
-        s = self.CONSTANTS
-        a1 = C['a1']
-        if self.region == 'jptw':
-            a1 += C['a1_del']
-        if pga:
-            r = 'tw'
-        else:
-            r = self.region
-        return a1 + C['a7'] + (C['a2'] + C['a14'] + s['a3'] * (mag - 7.8)) \
-            * np.log(rrup + s['c4'] * math.exp(s['a9'] * (mag - 6))) \
-            + C['a6' + r] * rrup
-
-    def _fztor(self, C, ztor):
-        """
-        Ztor factor for subduction intraslab.
-        """
-        return C['a11'] * (min(ztor, 80) - 40)
