@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: tabstop=4 shiftwidth=4 softtabstop=4
 #
-# Copyright (C) 2012-2021 GEM Foundation
+# Copyright (C) 2020-2021, GEM Foundation
 #
 # OpenQuake is free software: you can redistribute it and/or modify it
 # under the terms of the GNU Affero General Public License as published
@@ -14,7 +14,7 @@
 # GNU Affero General Public License for more details.
 #
 # You should have received a copy of the GNU Affero General Public License
-# along with OpenQuake. If not, see <http://www.gnu.org/licenses/>.
+# along with OpenQuake.  If not, see <http://www.gnu.org/licenses/>.
 
 """
 Module exports :class:`StewartEtAl2016`,
@@ -24,16 +24,13 @@ Module exports :class:`StewartEtAl2016`,
                :class:`StewartEtAl2016RegCHNNoSOF`,
                :class:`StewartEtAl2016RegJPNNoSOF`,
 """
-
-import numpy as np
-
-from openquake.hazardlib.gsim.base import GMPE, CoeffsTable
+from openquake.hazardlib.gsim.base import CoeffsTable, add_alias
 from openquake.hazardlib.gsim.boore_2014 import BooreEtAl2014
 from openquake.hazardlib import const
 from openquake.hazardlib.imt import PGA, PGV, SA
 
 
-class StewartEtAl2016(GMPE):
+class StewartEtAl2016(BooreEtAl2014):
     """
     Implements the SBSA15 GMPE by Stewart et al. (2016) for vertical-component
     ground motions from the PEER NGA-West2 Project.
@@ -44,22 +41,22 @@ class StewartEtAl2016(GMPE):
     Note that this is a more updated version than the GMPE described in the
     original PEER Report 2013/24.
 
-    **Reference:**
+    Reference:
 
     Stewart, J., Boore, D., Seyhan, E., & Atkinson, G. (2016). NGA-West2
     Equations for Predicting Vertical-Component PGA, PGV, and 5%-Damped PSA
-    from Shallow Crustal Earthquakes. *Earthquake Spectra*, *32*(2), 1005-1031.
+    from Shallow Crustal Earthquakes. Earthquake Spectra, 32(2), 1005-1031.
     """
+    region = "CAL"
+
+    kind = "stewart"
+
     #: Supported tectonic region type is active shallow crust; see title.
     DEFINED_FOR_TECTONIC_REGION_TYPE = const.TRT.ACTIVE_SHALLOW_CRUST
 
     #: Supported intensity measure types are spectral acceleration,
     #: peak ground velocity and peak ground acceleration; see title.
-    DEFINED_FOR_INTENSITY_MEASURE_TYPES = set([
-        PGA,
-        PGV,
-        SA
-    ])
+    DEFINED_FOR_INTENSITY_MEASURE_TYPES = {PGA, PGV, SA}
 
     #: Supported intensity measure component is the
     #: :attr:`~openquake.hazardlib.const.IMC.Vertical` direction component;
@@ -68,11 +65,8 @@ class StewartEtAl2016(GMPE):
 
     #: Supported standard deviation types are inter-event, intra-event
     #: and total; see the section for "Aleatory-Uncertainty Function".
-    DEFINED_FOR_STANDARD_DEVIATION_TYPES = set([
-        const.StdDev.TOTAL,
-        const.StdDev.INTER_EVENT,
-        const.StdDev.INTRA_EVENT
-    ])
+    DEFINED_FOR_STANDARD_DEVIATION_TYPES = {
+        const.StdDev.TOTAL, const.StdDev.INTER_EVENT, const.StdDev.INTRA_EVENT}
 
     #: Required site parameter is Vs30
     REQUIRES_SITES_PARAMETERS = {'vs30'}
@@ -83,133 +77,12 @@ class StewartEtAl2016(GMPE):
     #: Required distance measure is Rjb
     REQUIRES_DISTANCES = {'rjb'}
 
-    def get_mean_and_stddevs(self, sites, rup, dists, imt, stddev_types):
-        """
-        See :meth:`superclass method
-        <.base.GroundShakingIntensityModel.get_mean_and_stddevs>`
-        for spec of input and result values.
-        """
-        # extracting dictionary of coefficients specific to required
-        # intensity measure type.
-        C = self.COEFFS[imt]
-        C_PGA = self.COEFFS[PGA()]
-        # Retrieve PGAr case
-        pga_rock = self._get_pga_on_rock(C_PGA, rup, dists)
-        # SBSA15 Functional Form, Equation 1 (in natural log units)
-        mean = (self._get_magnitude_scaling_term(C, rup) +
-                self._get_path_scaling(C, dists.rjb, rup.mag) +
-                self._get_site_scaling(C, pga_rock, sites, dists.rjb))
-        # SBSA15 Std Dev Model, Equations 9 to 11 (in natural log units)
-        stddevs = self._get_stddevs(C, rup, sites, stddev_types)
-        return mean, stddevs
+    REQUIRES_ATTRIBUTES = {'region', 'sof'}
 
-    def _get_pga_on_rock(self, C, rup, dists):
-        """
-        Returns the median PGA on rock (in g units), PGAr, which is a sum of
-        the magnitude and distance scaling. Precisely, it is obtained by
-        evaluating Equation 1 for the given magnitude, fault mechanism, and Rjb
-        with Vs30=760 m∕s.
-        """
-        return np.exp(self._get_magnitude_scaling_term(C, rup) +
-                      self._get_path_scaling(C, dists.rjb, rup.mag))
-
-    def _get_magnitude_scaling_term(self, C, rup):
-        """
-        Returns the magnitude scaling term component, defined in Equation 2
-        """
-        return BooreEtAl2014._get_magnitude_scaling_term(self, C, rup)
-
-    def _get_style_of_faulting_term(self, C, rup):
-        """
-        Returns the style-of-faulting term component, defined in Equation 2
-        """
-        return BooreEtAl2014._get_style_of_faulting_term(self, C, rup)
-
-    def _get_path_scaling(self, C, rjb, mag):
-        """
-        Returns the path scaling term defined in Equation 3
-        """
-        # Calculate R in Equation 4
-        rval = np.sqrt((rjb ** 2.0) + (C["h"] ** 2.0))
-        delta_c3 = self._get_deltac3(C)
-
-        # Calculate geometric spreading component of path scaling term
-        fp_geom = ((C["c1"] + C["c2"] * (mag - self.CONSTS["Mref"])) *
-                   np.log(rval / self.CONSTS["Rref"]))
-        # Calculate anelastic attenuation component of path scaling term, with
-        # delta c3 accounting for regional effects
-        fp_atten = (C["c3"] + delta_c3) * (rval - self.CONSTS["Rref"])
-        return fp_geom + fp_atten
-
-    def _get_deltac3(self, C):
-        """
-        Returns the regional dependent delta_c3 in the path scaling term in
-        Equation 3
-
-        We assume California as the default region or average Q case, hence
-        here the regional delta c3 is assumed = 0.
-        """
-        return 0.
-
-    def _get_site_scaling(self, C, pga_rock, sites, rjb):
-        """
-        Returns the site-scaling term in Equation 5, broken down into a
-        linear and a nonlinear component
-
-        Unlike in BSSA14, basin depth effects are not accounted for in this
-        model.
-        """
-        flin = BooreEtAl2014._get_linear_site_term(self, C, sites.vs30)
-        fnl = BooreEtAl2014._get_nonlinear_site_term(self, C,
-                                                     sites.vs30, pga_rock)
-        return flin + fnl
-
-    def _get_stddevs(self, C, rup, sites, stddev_types):
-        """
-        Returns the aleatory uncertainty terms described in Equations 9 to 11
-        """
-        stddevs = []
-        num_sites = len(sites.vs30)
-        tau = self._get_inter_event_tau(C, rup.mag)
-        phi = self._get_intra_event_phi(C, rup.mag)
-        for stddev_type in stddev_types:
-            assert stddev_type in self.DEFINED_FOR_STANDARD_DEVIATION_TYPES
-            if stddev_type == const.StdDev.TOTAL:
-                stddevs.append(np.sqrt((tau ** 2.0) + (phi ** 2.0)) +
-                               np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTRA_EVENT:
-                stddevs.append(phi + np.zeros(num_sites))
-            elif stddev_type == const.StdDev.INTER_EVENT:
-                stddevs.append(tau + np.zeros(num_sites))
-        # return std dev values for each stddev type in site collection
-        return stddevs
-
-    def _get_inter_event_tau(self, C, mag):
-        """
-        Returns the inter-event standard deviation (tau), which is dependent
-        on magnitude
-        """
-        if mag <= 4.5:
-            tau = C["tau1"]
-        elif mag >= 5.5:
-            tau = C["tau2"]
-        else:
-            tau = C["tau1"] + (C["tau2"] - C["tau1"]) * (mag - 4.5)
-        return tau
-
-    def _get_intra_event_phi(self, C, mag):
-        """
-        Returns the intra-event standard deviation (phi)
-
-        In SBSA15, the intra-event standard deviation only depends on magnitude
-        """
-        if mag <= 4.5:
-            phi = C["phi1"]
-        elif mag >= 5.5:
-            phi = C["phi2"]
-        else:
-            phi = (C["phi1"] + (C["phi2"] - C["phi1"]) * (mag - 4.5))
-        return phi
+    def __init__(self, region='CAL', sof=True, **kwargs):
+        super().__init__(**kwargs)
+        self.region = region
+        self.sof = sof
 
     #: Table of period-dependent regression coefficients obtained from the
     #: supplementary material in EQS paper
@@ -324,87 +197,9 @@ class StewartEtAl2016(GMPE):
     10.00    -4.870000    -4.989000    -4.795000    -4.668000    2.187000    0.000000     1.158000     6.200000    -1.015000    0.160100    0.000000     0.000896    0.000000     5.930000    -0.276700    775.000000     0.000000     -0.001360    0.48361    0.41840    0.37531    0.58136
     """)
 
-    #: Equation constants that are IMT-independent
-    CONSTS = {
-        "Mref": 4.5,
-        "Rref": 1.0,
-        "Vref": 760.0,
-        "f1": 0.0,
-        "f3": 0.1}
 
-
-class StewartEtAl2016RegCHN(StewartEtAl2016):
-    """
-    This class implements the Stewart et al. (2016) model considering the
-    correction to the path scaling term for High Q regions (e.g. China)
-    The modification is made to the "Dc3" coefficient
-    """
-    def _get_deltac3(self, C):
-        """
-        Returns the regional dependent delta_c3 in the path scaling term in
-        Equation 3
-        """
-        return C['Dc3CH']
-
-
-class StewartEtAl2016RegJPN(StewartEtAl2016):
-    """
-    This class implements the Stewart et al. (2016) model considering the
-    correction to the path scaling term for Low Q regions (e.g. Japan)
-    The modification is made to the "Dc3" coefficient
-    """
-    def _get_deltac3(self, C):
-        """
-        Returns the regional dependent delta_c3 in the path scaling term in
-        Equation 3
-        """
-        return C['Dc3JP']
-
-
-class StewartEtAl2016NoSOF(StewartEtAl2016):
-    """
-    The Stewart et al. (2016) GMPE can consider the case in which the
-    style-of-faulting is unspecified. In this case the GMPE is no longer
-    dependent on rake.
-    """
-    #: Required rupture parameter is magnitude
-    REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-
-    def _get_style_of_faulting_term(self, C, rup):
-        """
-        Returns the coefficient of the "Unspecified" style-of-faulting
-        """
-        return C["e0"]
-
-
-class StewartEtAl2016RegCHNNoSOF(StewartEtAl2016RegCHN):
-    """
-    The Stewart et al. (2016) GMPE, implemented for High Q regional datasets,
-    (e.g. China) for the case in which the style-of-faulting is
-    unspecified. In this case the GMPE is no longer
-    dependent on rake.
-    """
-    #: Required rupture parameter is magnitude
-    REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-
-    def _get_style_of_faulting_term(self, C, rup):
-        """
-        Returns the coefficient of the "Unspecified" style-of-faulting
-        """
-        return C["e0"]
-
-
-class StewartEtAl2016RegJPNNoSOF(StewartEtAl2016RegJPN):
-    """
-    The Stewart et al. (2016) GMPE, implemented for Low Q regional datasets,
-    (e.g. Japan) for the case in which the style-of-faulting is unspecified.
-    In this case the GMPE is no longer dependent on rake.
-    """
-    #: Required rupture parameters are magnitude
-    REQUIRES_RUPTURE_PARAMETERS = {'mag'}
-
-    def _get_style_of_faulting_term(self, C, rup):
-        """
-        Returns the coefficient of the "Unspecified" style-of-faulting
-        """
-        return C["e0"]
+add_alias('StewartEtAl2016RegCHN', StewartEtAl2016, region="CHN")
+add_alias('StewartEtAl2016RegJPN', StewartEtAl2016, region="JPN")
+add_alias('StewartEtAl2016NoSOF', StewartEtAl2016, sof=0)
+add_alias('StewartEtAl2016CHNNoSOF', StewartEtAl2016, region="CHN", sof=0)
+add_alias('StewartEtAl2016JPNNoSOF', StewartEtAl2016, region="JPN", sof=0)
