@@ -24,6 +24,8 @@ import re
 import collections
 import numpy
 
+FREQUENCY_PATTERN = '^(EAS|FAS|DRVT)\\((\\d+\\.*\\d*)\\)'
+
 
 def positivefloat(val):
     """
@@ -64,7 +66,16 @@ def from_string(imt, _damping=5.0):
     :param str imt:
         Intensity Measure Type.
     """
-    if re.match(r'[ \+\d\.]+', imt):
+    m = re.match(FREQUENCY_PATTERN, imt)
+    if m:
+        if m.group(1) == 'EAS':
+            im = EAS(float(m.group(2)))
+        elif m.group(1) == 'FAS':
+            im = FAS(float(m.group(2)))
+        elif m.group(1) == 'DRVT':
+            im = DRVT(float(m.group(2)))
+        return im
+    elif re.match(r'[ \+\d\.]+', imt):
         return SA(float(imt))
     return IMT(*imt2tup(imt))
 
@@ -82,6 +93,7 @@ IMT.__gt__ = lambda self, other: self[1] > other[1]
 IMT.__le__ = lambda self, other: self[1] <= other[1]
 IMT.__ge__ = lambda self, other: self[1] >= other[1]
 IMT.__repr__ = repr
+IMT.frequency = property(lambda self: 1. / self.period)
 
 
 def PGA():
@@ -106,6 +118,30 @@ def PGD():
     return IMT('PGD')
 
 
+def EAS(frequency):
+    """
+    Effective Amplitude Spectrum in terms of a frequency (in Hz).
+    """
+    period = 1. / frequency
+    return IMT('EAS(%s)' % period, period, 5.0)
+
+
+def FAS(frequency):
+    """
+    Fourier Amplitude Spectrum in terms of a frequency (in Hz).
+    """
+    period = 1. / frequency
+    return IMT('FAS(%s)' % period, period, 5.0)
+
+
+def DRVT(frequency):
+    """
+    Duration as defined in Bora et al. (2019)
+    """
+    period = 1. / frequency
+    return IMT('DRVT(%s)' % period, period, 5.0)
+
+
 def SA(period, damping=5.0):
     """
     Spectral acceleration, defined as the maximum acceleration of a damped,
@@ -145,10 +181,10 @@ def RSD():
     Relative significant duration, 5-95% of :class:`Arias intensity<IA>`,
     in seconds.
     """
-    return IMT('RDS')
+    return IMT('RSD')
 
 
-def RSD595(IMT):
+def RSD595():
     """
     Alias for RSD
     """
