@@ -97,6 +97,16 @@ class DuplicatedID(Exception):
     """Raised when two sources with the same ID are found in a source model"""
 
 
+def check_unique(ids):
+    """
+    Raise a DuplicatedID exception if there are duplicated IDs
+    """
+    unique, counts = numpy.unique(ids, return_counts=True)
+    for u, c in zip(unique, counts):
+        if c > 1:
+            raise DuplicatedID(u)
+
+
 class SourceModel(collections.abc.Sequence):
     """
     A container of source groups with attributes name, investigation_time
@@ -147,6 +157,16 @@ class SourceModel(collections.abc.Sequence):
             self.src_groups.append(grp)
 
 
+class GeometryModel(object):
+    """
+    Contains a dictionary of sections
+    """
+    def __init__(self, sections):
+        check_unique(sections)
+        self.sections = sections
+        self.src_groups = []
+
+
 def get_tag_version(nrml_node):
     """
     Extract from a node of kind NRML the tag and the version. For instance
@@ -169,14 +189,18 @@ def to_python(fname, *args):
 node_to_obj = CallableDict(keyfunc=get_tag_version, keymissing=lambda n, f: n)
 # dictionary of functions with at least two arguments, node and fname
 
+default = sourceconverter.SourceConverter(area_source_discretization=10,
+                                          rupture_mesh_spacing=10)
+
 
 @node_to_obj.add(('ruptureCollection', 'nrml/0.5'))
 def get_rupture_collection(node, fname, converter):
     return converter.convert_node(node)
 
 
-default = sourceconverter.SourceConverter(area_source_discretization=10,
-                                          rupture_mesh_spacing=10)
+@node_to_obj.add(('geometryModel', 'nrml/0.5'))
+def get_geometry_model(node, fname, converter):
+    return GeometryModel(converter.convert_node(node))
 
 
 @node_to_obj.add(('sourceModel', 'nrml/0.4'))
@@ -412,7 +436,7 @@ def get(xml, investigation_time=50., rupture_mesh_spacing=5.,
         width_of_mfd_bin=width_of_mfd_bin,
         area_source_discretization=area_source_discretization)
     src = conv.convert_node(node)
-    src.id = 0
+    src.grp_id = src.id = 0
     return src
 
 
