@@ -96,6 +96,16 @@ class CharacteristicFaultSource(ParametricSeismicSource):
                 mag, self.rake, self.tectonic_region_type, hypocenter,
                 self.surface, occurrence_rate, self.temporal_occurrence_model)
 
+    def few_ruptures(self):
+        """
+        Fast version of iter_ruptures used in estimate_weight
+        """
+        hypocenter = self.surface.get_middle_point()
+        mag, rate = list(self.get_annual_occurrence_rates())[-1]
+        yield ParametricProbabilisticRupture(
+            mag, self.rake, self.tectonic_region_type, hypocenter,
+            self.surface, rate, self.temporal_occurrence_model)
+
     def count_ruptures(self):
         """
         See :meth:
