@@ -18,6 +18,7 @@ Module :mod:`openquake.hazardlib.source.non_parametric` defines
 :class:`NonParametricSeismicSource`
 """
 import numpy
+from openquake.baselib.general import block_splitter
 from openquake.hazardlib.source.base import BaseSeismicSource
 from openquake.hazardlib.geo.surface.gridded import GriddedSurface
 from openquake.hazardlib.geo.surface.multi import MultiSurface
@@ -76,17 +77,26 @@ class NonParametricSeismicSource(BaseSeismicSource):
                     rup.mag, rup.rake, self.tectonic_region_type,
                     rup.hypocenter, rup.surface, pmf, weight=rup.weight)
 
+    def few_ruptures(self):
+        """
+        Fast version of iter_ruptures used in estimate_weight
+        """
+        for i, (rup, pmf) in enumerate(self.data):
+            if i % 25 == 0 and rup.mag >= self.min_mag:
+                yield NonParametricProbabilisticRupture(
+                    rup.mag, rup.rake, self.tectonic_region_type,
+                    rup.hypocenter, rup.surface, pmf, weight=rup.weight)
+
     def __iter__(self):
         if len(self.data) == 1:  # there is nothing to split
             yield self
             return
-        for i, rup_pmf in enumerate(self.data):
+        for i, block in enumerate(block_splitter(self.data, 100)):
             source_id = '%s:%d' % (self.source_id, i)
             src = self.__class__(source_id, self.name,
-                                 self.tectonic_region_type, [rup_pmf])
-            src.num_ruptures = 1
+                                 self.tectonic_region_type, block)
+            src.num_ruptures = len(block)
             src.trt_smr = self.trt_smr
-            src.id = self.id
             yield src
 
     def count_ruptures(self):
@@ -180,7 +190,22 @@ class NonParametricSeismicSource(BaseSeismicSource):
             i += 1
 
     def __repr__(self):
-        return '<%s gridded=%s>' % (self.__class__.__name__, self.is_gridded())
+        return '<%s %s gridded=%s>' % (
+            self.__class__.__name__, self.source_id, self.is_gridded())
+
+    @property
+    def mesh_size(self):
+        """
+        :returns: the number of points in the underlying mesh
+        """
+        n = 0
+        for rup, pmf in self.data:
+            if isinstance(rup.surface, MultiSurface):
+                for sfc in rup.surface.surfaces:
+                    n += len(sfc.mesh)
+            else:
+                n += len(rup.surface.mesh)
+        return n
 
     @property
     def polygon(self):
