@@ -73,7 +73,8 @@ rupture_dt = numpy.dtype([
     ('trt_smr', U16),
     ('code', U8),
     ('n_occ', U32),
-    ('mag', F32), ('rake', F32),
+    ('mag', F32),
+    ('rake', F32),
     ('occurrence_rate', F32),
     ('minlon', F32),
     ('minlat', F32),
@@ -561,13 +562,13 @@ class ParametricProbabilisticRupture(BaseRupture):
 class PointSurface:
     """
     A fake surface used in PointRuptures.
-    The parameters `hypocenter`, `strike` and `dip` are determined by
+    The parameters `hypocenter` and `strike` are determined by
     collapsing the corresponding parameters in the original PointSource.
     """
-    def __init__(self, hypocenter, strike, dip):
+    def __init__(self, hypocenter, strike):
         self.hypocenter = hypocenter
         self.strike = strike
-        self.dip = dip
+        self.dip = 90.
 
     def get_strike(self):
         return self.strike
@@ -594,16 +595,15 @@ class PointRupture(ParametricProbabilisticRupture):
     size effects can be neglected.
     """
     def __init__(self, mag, tectonic_region_type, hypocenter, strike,
-                 dip, rake, occurrence_rate, temporal_occurrence_model):
+                 rake, occurrence_rate, temporal_occurrence_model):
         self.tectonic_region_type = tectonic_region_type
         self.hypocenter = hypocenter
         self.mag = mag
         self.strike = strike
         self.rake = rake
-        self.dip = dip
         self.occurrence_rate = occurrence_rate
         self.temporal_occurrence_model = temporal_occurrence_model
-        self.surface = PointSurface(hypocenter, strike, dip)
+        self.surface = PointSurface(hypocenter, strike)
         self.weight = None  # no mutex
 
 
@@ -766,23 +766,11 @@ class RuptureProxy(object):
     A proxy for a rupture record.
 
     :param rec: a record with the rupture parameters
-    :param nsites: approx number of sites affected by the rupture
     """
-    def __init__(self, rec, nsites=None, scenario=False):
+    def __init__(self, rec, scenario=False):
         self.rec = rec
-        self.nsites = nsites
         self.scenario = scenario
 
-    @property
-    def weight(self):
-        """
-        :returns:
-            heuristic weight for the underlying rupture, depending on the
-            number of occurrences and number of presumably affected sites
-        """
-        return self['n_occ'] * (
-            10 if self.nsites is None else max(self.nsites, 10))
-
     def __getitem__(self, name):
         return self.rec[name]
 
@@ -793,11 +781,15 @@ class RuptureProxy(object):
         """
         # not implemented: rupture_slip_direction
         rupture = _get_rupture(self.rec, self.geom, trt)
-        ebr = EBRupture(rupture, self.rec['source_id'], self.rec['trt_smr'],
-                        self.rec['n_occ'], self.rec['id'], self.rec['e0'],
-                        self.scenario)
+        ebr = EBRupture(rupture, self['source_id'], self['trt_smr'],
+                        self['n_occ'], self['id'], self['e0'], self.scenario)
         return ebr
 
+    def __repr__(self):
+        src = self['source_id'].decode('ascii')
+        return '<%s#%d[%s], w=%d>' % (self.__class__.__name__,
+                                      self['id'], src, self['n_occ'])
+
 
 def get_ruptures(fname_csv):
     """
