@@ -22,7 +22,7 @@ import numpy
 import pandas
 from scipy.stats import norm
 from scipy.special import ndtr
-from openquake.baselib.general import agg_probs
+from openquake.baselib.general import AccumDict, agg_probs
 
 
 def _truncnorm_sf(truncation_level, values):
@@ -267,6 +267,28 @@ def compute_pmap_stats(pmaps, stats, weights, imtls):
     return out
 
 
+def calc_stats(df, kfields, stats, weights):
+    """
+    :param df: a pandas DataFrame with a column rlz_id
+    :param kfields: fields used in the group by
+    :param stats: a dictionary stat_name->stat_func
+    :param weights: an array of weights for each realization
+    :returns: a DataFrame with the statistics
+    """
+    acc = AccumDict(accum=[])
+    vfields = [f for f in df.columns if f not in kfields and f != 'rlz_id']
+    for key, group in df.groupby(kfields):
+        ws = weights[group.rlz_id]
+        for name, func in stats.items():
+            for k, kf in zip(key, kfields):
+                acc[kf].append(k)
+            for vf in vfields:
+                v = apply_stat(func, getattr(group, vf).to_numpy(), ws)
+                acc[vf].append(v)
+            acc['stat'].append(name)
+    return pandas.DataFrame(acc)
+
+
 # NB: this is a function linear in the array argument
 def compute_stats(array, stats, weights):
     """
