@@ -25,15 +25,16 @@ import re
 import ast
 import json
 import toml
+import socket
 import logging
 import numpy
 
 from openquake.baselib.general import distinct
-from openquake.baselib import hdf5
+from openquake.baselib import config, hdf5
 from openquake.hazardlib import imt, scalerel, gsim, pmf, site
 from openquake.hazardlib.gsim.base import registry, gsim_aliases
 from openquake.hazardlib.calc import disagg
-from openquake.hazardlib.calc.filters import MagDepDistance, floatdict  # needed
+from openquake.hazardlib.calc.filters import IntegrationDistance, floatdict  # needed
 
 PRECISION = pmf.PRECISION
 
@@ -64,6 +65,7 @@ class FromFile(object):
     external file and not computed with a GSIM.
     """
     DEFINED_FOR_INTENSITY_MEASURE_TYPES = set()
+    REQUIRES_RUPTURE_PARAMETERS = set()
     REQUIRES_SITES_PARAMETERS = set()
     REQUIRES_DISTANCES = set()
     DEFINED_FOR_REFERENCE_VELOCITY = None
@@ -72,6 +74,9 @@ class FromFile(object):
     def init(self):
         pass
 
+    def compute(self, ctx, imts, mean, sig, tau, phi):
+        pass
+
     def __repr__(self):
         return '[FromFile]'
 
@@ -130,11 +135,7 @@ def gsim(value, basedir=''):
         gsim_class = registry[gsim_name]
     except KeyError:
         raise ValueError('Unknown GSIM: %s' % gsim_name)
-    if basedir:
-        gs = gsim_class(**kwargs)
-    else:
-        gs = object.__new__(gsim_class)
-        gs.kwargs = kwargs
+    gs = gsim_class(**kwargs)
     gs._toml = '\n'.join(line.strip() for line in value.splitlines())
     return gs
 
@@ -1057,6 +1058,21 @@ def uncertainty_model(value):
     return ' '.join(value.split())  # remove newlines too
 
 
+def host_port(value=None):
+    """
+    Returns a pair (host_IP, port_number).
+
+    >>> host_port('localhost:1908')
+    ('127.0.0.1', 1908)
+
+    If value is missing returns the parameters in openquake.cfg
+    """
+    if not value:
+        return (config.dbserver.host, config.dbserver.port)
+    host, port = value.split(':')
+    return socket.gethostbyname(host), int(port)
+
+
 # used for the exposure validation
 cost_type = Choice('structural', 'nonstructural', 'contents',
                    'business_interruption')
